# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Parentage__get_spanners_01():
    '''Get spanners in proper parentage.
    '''

    container = Container("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(Beam(), container[:])
    slur = Slur()
    attach(slur, container[:])
    trill = spannertools.TrillSpanner()
    attach(trill, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8 [ ( \startTrillSpan
            d'8
            e'8
            f'8 ] ) \stopTrillSpan
        }
        '''
        )

    spanners = container[0]._get_parentage()._get_spanners()
    spanners == set([beam, slur, trill])
    spanners = container._get_parentage()._get_spanners()
    spanners == set([trill])


def test_selectiontools_Parentage__get_spanners_02():
    '''Get spanners in improper parentage.
    '''

    container = Container("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, container[:])
    slur = Slur()
    attach(slur, container[:])
    trill = spannertools.TrillSpanner()
    attach(trill, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8 [ ( \startTrillSpan
            d'8
            e'8
            f'8 ] ) \stopTrillSpan
        }
        '''
        )

    parentage = container._get_parentage(include_self=False)
    assert parentage._get_spanners() == set([])


# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_TimeSignature___init___01():
    r'''Initializes time signature from integer pair.
    '''

    time_signature = TimeSignature((9, 32))
    assert isinstance(time_signature, TimeSignature)
    assert format(time_signature, 'lilypond') == '\\time 9/32'



# -*- coding: utf-8 -*-


def is_integer_equivalent_pair(argument):
    r'''Is true when `argument` is a pair of integer-equivalent items.
    Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.is_integer_equivalent_pair((2.0, '3'))
            True

        ::

            >>> mathtools.is_integer_equivalent_pair((2.5, '3'))
            False

    Returns true or false.
    '''
    from abjad.tools import mathtools
    return (
        isinstance(argument, tuple) and
        len(argument) == 2 and
        all(mathtools.is_integer_equivalent(x) for x in argument)
        )



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_quantizationtools_QGridLeaf___copy___01():
    leaf = quantizationtools.QGridLeaf(1)
    copied = copy.copy(leaf)
    assert format(leaf) == format(copied)
    assert leaf != copied
    assert leaf is not copied


def test_quantizationtools_QGridLeaf___copy___02():
    leaf = quantizationtools.QGridLeaf(2, [
        quantizationtools.QEventProxy(quantizationtools.SilentQEvent(1000), 0.5)
        ])
    copied = copy.copy(leaf)
    assert format(leaf) == format(copied)
    assert leaf != copied
    assert leaf is not copied



# -*- coding: utf-8 -*-
from abjad.tools import abctools
from abjad.tools import durationtools
from abjad.tools import sequencetools


class MeterManager(abctools.AbjadObject):
    r'''A meter manager.
    '''

    ### PUBLIC METHODS ###

    @staticmethod
    def get_offsets_at_depth(depth, offset_inventory):
        r'''Gets offsets at `depth` in `offset_inventory`.
        '''
        if depth < len(offset_inventory):
            return offset_inventory[depth]
        while len(offset_inventory) <= depth:
            new_offsets = []
            old_offsets = offset_inventory[-1]
            for first, second in sequencetools.Sequence(old_offsets).nwise():
                new_offsets.append(first)
                difference = second - first
                half = (first + second) / 2
                if durationtools.Duration(1, 8) < difference:
                    new_offsets.append(half)
                else:
                    one_quarter = (first + half) / 2
                    three_quarters = (half + second) / 2
                    new_offsets.append(one_quarter)
                    new_offsets.append(half)
                    new_offsets.append(three_quarters)
            new_offsets.append(old_offsets[-1])
            offset_inventory.append(tuple(new_offsets))
        return offset_inventory[depth]

    @staticmethod
    def is_acceptable_logical_tie(
        logical_tie_duration=None,
        logical_tie_starts_in_offsets=None,
        logical_tie_stops_in_offsets=None,
        maximum_dot_count=None,
        ):
        r'''Is true if logical tie is acceptable.
        '''
        #print '\tTESTING ACCEPTABILITY'
        if not logical_tie_duration.is_assignable:
            return False
        if maximum_dot_count is not None and \
            maximum_dot_count < logical_tie_duration.dot_count:
            return False
        if not logical_tie_starts_in_offsets and \
            not logical_tie_stops_in_offsets:
            return False
        return True

    @staticmethod
    def is_boundary_crossing_logical_tie(
        boundary_depth=None,
        boundary_offsets=None,
        logical_tie_start_offset=None,
        logical_tie_stop_offset=None,
        ):
        r'''Is true if logical tie crosses meter boundaries.
        '''
        #print '\tTESTING BOUNDARY CROSSINGS'
        if boundary_depth is None:
            return False
        if not any(logical_tie_start_offset < x < logical_tie_stop_offset
            for x in boundary_offsets):
            return False
        if (logical_tie_start_offset in boundary_offsets and
            logical_tie_stop_offset in boundary_offsets):
            return False
        return True

    @staticmethod
    def iterate_rewrite_inputs(argument):
        r'''Iterates topmost masked logical ties, rest groups and containers
        in `argument`, masked by `argument`.

        ::

            >>> from abjad.tools import metertools
            >>> from abjad.tools import scoretools

        ::

            >>> string = "abj: ! 2/4 c'4 d'4 ~ !"
            >>> string += "! 4/4 d'8. r16 r8. e'16 ~ "
            >>> string += "2/3 { e'8 ~ e'8 f'8 ~ } f'4 ~ !"
            >>> string += "! 4/4 f'8 g'8 ~ g'4 a'4 ~ a'8 b'8 ~ !"
            >>> string += "! 2/4 b'4 c''4 !"
            >>> string = string.replace('!', '|')
            >>> staff = scoretools.Staff(string)

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                {
                    \time 2/4
                    c'4
                    d'4 ~
                }
                {
                    \time 4/4
                    d'8.
                    r16
                    r8.
                    e'16 ~
                    \times 2/3 {
                        e'8 ~
                        e'8
                        f'8 ~
                    }
                    f'4 ~
                }
                {
                    f'8
                    g'8 ~
                    g'4
                    a'4 ~
                    a'8
                    b'8 ~
                }
                {
                    \time 2/4
                    b'4
                    c''4
                }
            }

        ::

            >>> for x in metertools.MeterManager.iterate_rewrite_inputs(
            ...     staff[0]): x
            ...
            LogicalTie([Note("c'4")])
            LogicalTie([Note("d'4")])

        ::

            >>> for x in metertools.MeterManager.iterate_rewrite_inputs(
            ...     staff[1]): x
            ...
            LogicalTie([Note("d'8.")])
            LogicalTie([Rest('r16'), Rest('r8.')])
            LogicalTie([Note("e'16")])
            Tuplet(Multiplier(2, 3), "e'8 ~ e'8 f'8 ~")
            LogicalTie([Note("f'4")])

        ::

            >>> for x in metertools.MeterManager.iterate_rewrite_inputs(
            ...     staff[2]): x
            ...
            LogicalTie([Note("f'8")])
            LogicalTie([Note("g'8"), Note("g'4")])
            LogicalTie([Note("a'4"), Note("a'8")])
            LogicalTie([Note("b'8")])

        ::

            >>> for x in metertools.MeterManager.iterate_rewrite_inputs(
            ...     staff[3]): x
            ...
            LogicalTie([Note("b'4")])
            LogicalTie([Note("c''4")])

        Returns generator.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        from abjad.tools import spannertools

        last_tie_spanner = None
        current_leaf_group = None
        current_leaf_group_is_silent = False

        for x in argument:
            if isinstance(x, (scoretools.Note, scoretools.Chord)):
                this_tie_spanner = x._get_spanners(spannertools.Tie) or None
                if current_leaf_group is None:
                    current_leaf_group = []
                elif current_leaf_group_is_silent or \
                    this_tie_spanner is None or \
                    last_tie_spanner != this_tie_spanner:
                    yield selectiontools.LogicalTie(current_leaf_group)
                    current_leaf_group = []
                current_leaf_group_is_silent = False
                current_leaf_group.append(x)
                last_tie_spanner = this_tie_spanner
            elif isinstance(x, (scoretools.Rest, scoretools.Skip)):
                if current_leaf_group is None:
                    current_leaf_group = []
                elif not current_leaf_group_is_silent:
                    yield selectiontools.LogicalTie(current_leaf_group)
                    current_leaf_group = []
                current_leaf_group_is_silent = True
                current_leaf_group.append(x)
                last_tie_spanner = None
            elif isinstance(x, scoretools.Container):
                if current_leaf_group is not None:
                    yield selectiontools.LogicalTie(current_leaf_group)
                    current_leaf_group = None
                    last_tie_spanner = None
                yield x

            else:
                message = 'unhandled component: {!r}.'
                message = message.format(x)
                raise Exception(message)
        if current_leaf_group is not None:
            yield selectiontools.LogicalTie(current_leaf_group)



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_InspectionAgent_get_leaf_01():

    staff = Staff([Voice("c'8 d'8 e'8 f'8"), Voice("g'8 a'8 b'8 c''8")])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \new Voice {
                c'8
                d'8
                e'8
                f'8
            }
            \new Voice {
                g'8
                a'8
                b'8
                c''8
            }
        }
        '''
        )

    leaves = select(staff).by_leaf()
    assert inspect_(leaves[0]).get_leaf(0) is leaves[0]
    assert inspect_(leaves[0]).get_leaf(1) is leaves[1]
    assert inspect_(leaves[0]).get_leaf(2) is leaves[2]
    assert inspect_(leaves[0]).get_leaf(3) is leaves[3]
    assert inspect_(leaves[0]).get_leaf(4) is None
    assert inspect_(leaves[0]).get_leaf(5) is None
    assert inspect_(leaves[0]).get_leaf(6) is None
    assert inspect_(leaves[0]).get_leaf(7) is None

    assert inspect_(leaves[0]).get_leaf(-1) is None


def test_agenttools_InspectionAgent_get_leaf_02():
    r'''Voice.
    '''

    voice = Voice([Note(i, (1, 8)) for i in range(4)])

    assert inspect_(voice[0]).get_leaf(1) is voice[1]
    assert inspect_(voice[1]).get_leaf(1) is voice[2]
    assert inspect_(voice[2]).get_leaf(1) is voice[3]
    assert inspect_(voice[3]).get_leaf(1) is None

    assert inspect_(voice[0]).get_leaf(-1) is None
    assert inspect_(voice[1]).get_leaf(-1) is voice[0]
    assert inspect_(voice[2]).get_leaf(-1) is voice[1]
    assert inspect_(voice[3]).get_leaf(-1) is voice[2]

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8
            cs'8
            d'8
            ef'8
        }
        '''
        )


def test_agenttools_InspectionAgent_get_leaf_03():
    r'''Staff.
    '''

    staff = Staff([Note(i, (1, 8)) for i in range(4)])

    assert inspect_(staff[0]).get_leaf(1) is staff[1]
    assert inspect_(staff[1]).get_leaf(1) is staff[2]
    assert inspect_(staff[2]).get_leaf(1) is staff[3]
    assert inspect_(staff[3]).get_leaf(1) is None

    assert inspect_(staff[0]).get_leaf(-1) is None
    assert inspect_(staff[1]).get_leaf(-1) is staff[0]
    assert inspect_(staff[2]).get_leaf(-1) is staff[1]
    assert inspect_(staff[3]).get_leaf(-1) is staff[2]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            cs'8
            d'8
            ef'8
        }
        '''
        )


def test_agenttools_InspectionAgent_get_leaf_04():
    r'''Container.
    '''

    container = Container([Note(i, (1, 8)) for i in range(4)])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            cs'8
            d'8
            ef'8
        }
        '''
        )

    assert inspect_(container[0]).get_leaf(1) is container[1]
    assert inspect_(container[1]).get_leaf(1) is container[2]
    assert inspect_(container[2]).get_leaf(1) is container[3]
    assert inspect_(container[3]).get_leaf(1) is None

    assert inspect_(container[0]).get_leaf(-1) is None
    assert inspect_(container[1]).get_leaf(-1) is container[0]
    assert inspect_(container[2]).get_leaf(-1) is container[1]
    assert inspect_(container[3]).get_leaf(-1) is container[2]


def test_agenttools_InspectionAgent_get_leaf_05():
    r'''Fixed-duration tuplet.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 cs'8 d'8")

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'8
            cs'8
            d'8
        }
        '''
        )

    assert inspect_(tuplet[0]).get_leaf(1) is tuplet[1]
    assert inspect_(tuplet[1]).get_leaf(1) is tuplet[2]
    assert inspect_(tuplet[2]).get_leaf(1) is None

    assert inspect_(tuplet[0]).get_leaf(-1) is None
    assert inspect_(tuplet[1]).get_leaf(-1) is tuplet[0]
    assert inspect_(tuplet[2]).get_leaf(-1) is tuplet[1]


def test_agenttools_InspectionAgent_get_leaf_06():
    r'''Contiguous containers inside a voice.
    '''

    container_1 = Container([Note(i, (1, 8)) for i in range(4)])
    container_2 = Container([Note(i, (1, 8)) for i in range(4, 8)])
    voice = Voice([container_1, container_2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8
                cs'8
                d'8
                ef'8
            }
            {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    assert inspect_(container_1[0]).get_leaf(1) is container_1[1]
    assert inspect_(container_1[1]).get_leaf(1) is container_1[2]
    assert inspect_(container_1[2]).get_leaf(1) is container_1[3]
    assert inspect_(container_1[3]).get_leaf(1) is container_2[0]

    assert inspect_(container_1[1]).get_leaf(-1) is container_1[0]
    assert inspect_(container_1[2]).get_leaf(-1) is container_1[1]
    assert inspect_(container_1[3]).get_leaf(-1) is container_1[2]
    assert inspect_(container_2[0]).get_leaf(-1) is container_1[3]


def test_agenttools_InspectionAgent_get_leaf_07():
    r'''Tuplets inside a voice.
    '''

    tuplet_1 = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 cs'8 d'8")
    tuplet_2 = scoretools.FixedDurationTuplet(Duration(2, 8), "ef'8 e'8 f'8")
    voice = Voice([tuplet_1, tuplet_2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8
                cs'8
                d'8
            }
            \times 2/3 {
                ef'8
                e'8
                f'8
            }
        }
        '''
        )

    assert inspect_(tuplet_1[0]).get_leaf(1) is tuplet_1[1]
    assert inspect_(tuplet_1[1]).get_leaf(1) is tuplet_1[2]
    assert inspect_(tuplet_1[2]).get_leaf(1) is tuplet_2[0]

    assert inspect_(tuplet_1[1]).get_leaf(-1) is tuplet_1[0]
    assert inspect_(tuplet_1[2]).get_leaf(-1) is tuplet_1[1]
    assert inspect_(tuplet_2[0]).get_leaf(-1) is tuplet_1[2]


def test_agenttools_InspectionAgent_get_leaf_08():
    r'''Does not continue across contiguous anonymous voices inside a staff.
    '''

    voice_1 = Voice([Note(i, (1, 8)) for i in range(4)])
    voice_2 = Voice([Note(i, (1, 8)) for i in range(4, 8)])
    staff = Staff([voice_1, voice_2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \new Voice {
                c'8
                cs'8
                d'8
                ef'8
            }
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    assert inspect_(voice_1[3]).get_leaf(1) is None
    assert inspect_(voice_2[0]).get_leaf(-1) is None


def test_agenttools_InspectionAgent_get_leaf_09():
    r'''Does cross contiguous equally named voices inside a staff.
    '''

    voice_1 = Voice([Note(i, (1, 8)) for i in range(4)])
    voice_1.name = 'My Voice'
    voice_2 = Voice([Note(i, (1, 8)) for i in range(4, 8)])
    voice_2.name = 'My Voice'
    staff = Staff([voice_1, voice_2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \context Voice = "My Voice" {
                c'8
                cs'8
                d'8
                ef'8
            }
            \context Voice = "My Voice" {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    assert inspect_(voice_1[0]).get_leaf(1) is voice_1[1]
    assert inspect_(voice_1[1]).get_leaf(1) is voice_1[2]
    assert inspect_(voice_1[2]).get_leaf(1) is voice_1[3]
    assert inspect_(voice_1[3]).get_leaf(1) is voice_2[0]

    assert inspect_(voice_1[1]).get_leaf(-1) is voice_1[0]
    assert inspect_(voice_1[2]).get_leaf(-1) is voice_1[1]
    assert inspect_(voice_1[3]).get_leaf(-1) is voice_1[2]
    assert inspect_(voice_2[0]).get_leaf(-1) is voice_1[3]


def test_agenttools_InspectionAgent_get_leaf_10():
    r'''Does not connect through contiguous unequally named voices.
    '''

    voice_1 = Voice([Note(i, (1, 8)) for i in range(4)])
    voice_1.name = 'Your Voice'
    voice_2 = Voice([Note(i, (1, 8)) for i in range(4, 8)])
    voice_2.name = 'My Voice'
    staff = Staff([voice_1, voice_2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \context Voice = "Your Voice" {
                c'8
                cs'8
                d'8
                ef'8
            }
            \context Voice = "My Voice" {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    assert inspect_(voice_1[0]).get_leaf(1) is voice_1[1]
    assert inspect_(voice_1[1]).get_leaf(1) is voice_1[2]
    assert inspect_(voice_1[2]).get_leaf(1) is voice_1[3]
    assert inspect_(voice_1[3]).get_leaf(1) is None

    voice_2.name = None
    assert inspect_(voice_1[3]).get_leaf(1) is None

    assert inspect_(voice_2[1]).get_leaf(-1) is voice_2[0]
    assert inspect_(voice_2[2]).get_leaf(-1) is voice_2[1]
    assert inspect_(voice_2[3]).get_leaf(-1) is voice_2[2]
    assert inspect_(voice_2[0]).get_leaf(-1) is None


def test_agenttools_InspectionAgent_get_leaf_11():
    r'''Does connect through like-named staves
    containing like-named voices.
    '''

    voice_1 = Voice([Note(i, (1, 8)) for i in range(4)])
    voice_1.name = 'low'
    voice_2 = Voice([Note(i, (1, 8)) for i in range(4, 8)])
    voice_2.name = 'low'

    staff_1 = Staff([voice_1])
    staff_1.name = 'mystaff'
    staff_2 = Staff([voice_2])
    staff_2.name = 'mystaff'

    container = Container([staff_1, staff_2])

    assert format(container) == stringtools.normalize(
        r'''
        {
            \context Staff = "mystaff" {
                \context Voice = "low" {
                    c'8
                    cs'8
                    d'8
                    ef'8
                }
            }
            \context Staff = "mystaff" {
                \context Voice = "low" {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
            }
        }
        '''
        )

    assert inspect_(voice_1[3]).get_leaf(1) is voice_2[0]
    assert inspect_(voice_2[0]).get_leaf(-1) is voice_1[3]


def test_agenttools_InspectionAgent_get_leaf_12():
    r'''Does connect through like-named staves containing
    like-named voices.
    '''

    lower_voice_1 = Voice([Note(i, (1, 8)) for i in range(4)])
    lower_voice_1.name = 'low'
    lower_voice_2 = Voice([Note(i, (1, 8)) for i in range(4 ,8)])
    lower_voice_2.name = 'low'
    higher_voice_1 = Voice([Note(i, (1 ,8)) for i in range(12,16)])
    higher_voice_1.name = 'high'
    higher_voice_2 = Voice([Note(i, (1 ,8)) for i in range(16,20)])
    higher_voice_2.name = 'high'

    staff_1 = Staff([higher_voice_1, lower_voice_1])
    staff_1.name = 'mystaff'
    staff_1.is_simultaneous = True
    staff_2 = Staff([lower_voice_2, higher_voice_2])
    staff_2.name = 'mystaff'
    staff_2.is_simultaneous = True

    container = Container([staff_1, staff_2])

    assert format(container) == stringtools.normalize(
        r'''
        {
            \context Staff = "mystaff" <<
                \context Voice = "high" {
                    c''8
                    cs''8
                    d''8
                    ef''8
                }
                \context Voice = "low" {
                    c'8
                    cs'8
                    d'8
                    ef'8
                }
            >>
            \context Staff = "mystaff" <<
                \context Voice = "low" {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
                \context Voice = "high" {
                    e''8
                    f''8
                    fs''8
                    g''8
                }
            >>
        }
        '''
        )

    assert inspect_(lower_voice_1[3]).get_leaf(1) is lower_voice_2[0]
    assert inspect_(higher_voice_1[3]).get_leaf(1) is higher_voice_2[0]

    assert inspect_(lower_voice_2[0]).get_leaf(-1) is lower_voice_1[3]
    assert inspect_(higher_voice_2[0]).get_leaf(-1) is higher_voice_1[3]


def test_agenttools_InspectionAgent_get_leaf_13():
    r'''Does connect through symmetrical nested containers in a voice.
    '''

    container_1 = Container([Note(i, (1 ,8)) for i in range(4)])
    container_1 = Container([container_1])
    container_2 = Container([Note(i, (1 ,8)) for i in range(4 ,8)])
    container_2 = Container([container_2])
    voice = Voice([container_1, container_2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                {
                    c'8
                    cs'8
                    d'8
                    ef'8
                }
            }
            {
                {
                    e'8
                    f'8
                    fs'8
                    g'8
                }
            }
        }
        '''
        )

    assert inspect_(container_1[0][0]).get_leaf(1) is container_1[0][1]
    assert inspect_(container_1[0][1]).get_leaf(1) is container_1[0][2]
    assert inspect_(container_1[0][2]).get_leaf(1) is container_1[0][3]
    assert inspect_(container_1[0][3]).get_leaf(1) is container_2[0][0]

    assert inspect_(container_2[0][1]).get_leaf(-1) is container_2[0][0]
    assert inspect_(container_2[0][2]).get_leaf(-1) is container_2[0][1]
    assert inspect_(container_2[0][3]).get_leaf(-1) is container_2[0][2]
    assert inspect_(container_2[0][0]).get_leaf(-1) is container_1[0][3]


def test_agenttools_InspectionAgent_get_leaf_14():
    r'''Tautological parentage asymmetries result in symmetric (balanced)
    logical voice parentage.
    '''

    container_1 = Container([Note(i, (1 ,8)) for i in range(4)])
    container_2 = Container([Note(i, (1 ,8)) for i in range(4 ,8)])
    container_2 = Container([container_2])
    container_2 = Container([container_2])
    voice = Voice([container_1, container_2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8
                cs'8
                d'8
                ef'8
            }
            {
                {
                    {
                        e'8
                        f'8
                        fs'8
                        g'8
                    }
                }
            }
        }
        '''
        )

    assert inspect_(container_1[0]).get_leaf(1) is container_1[1]
    assert inspect_(container_1[1]).get_leaf(1) is container_1[2]
    assert inspect_(container_1[2]).get_leaf(1) is container_1[3]
    assert inspect_(container_1[3]).get_leaf(1) is container_2[0][0][0]

    assert inspect_(container_2[0][0][1]).get_leaf(-1) is container_2[0][0][0]
    assert inspect_(container_2[0][0][2]).get_leaf(-1) is container_2[0][0][1]
    assert inspect_(container_2[0][0][3]).get_leaf(-1) is container_2[0][0][2]
    assert inspect_(container_2[0][0][0]).get_leaf(-1) is container_1[3]


def test_agenttools_InspectionAgent_get_leaf_15():
    r'''Tautological parentage asymmetries result in symmetric (balanced)
    lgoical voice parentage.
    '''

    container_1 = Container([Note(i, (1 ,8)) for i in range(4)])
    container_1 = Container([container_1])
    container_1 = Container([container_1])
    container_2 = Container([Note(i, (1 ,8)) for i in range(4 ,8)])
    voice = Voice([container_1, container_2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                {
                    {
                        c'8
                        cs'8
                        d'8
                        ef'8
                    }
                }
            }
            {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    assert inspect_(container_1[0][0][0]).get_leaf(1) is container_1[0][0][1]
    assert inspect_(container_1[0][0][1]).get_leaf(1) is container_1[0][0][2]
    assert inspect_(container_1[0][0][2]).get_leaf(1) is container_1[0][0][3]
    assert inspect_(container_1[0][0][3]).get_leaf(1) is container_2[0]

    assert inspect_(container_2[0]).get_leaf(-1) is container_1[0][0][3]
    assert inspect_(container_2[1]).get_leaf(-1) is container_2[0]
    assert inspect_(container_2[2]).get_leaf(-1) is container_2[1]
    assert inspect_(container_2[3]).get_leaf(-1) is container_2[2]


def test_agenttools_InspectionAgent_get_leaf_16():
    r'''Does connect in sequence of alternating containers and notes.
    '''

    container_1 = Container([Note(i, (1, 8)) for i in range(2)])
    container_2 = Container([Note(i, (1, 8)) for i in range(3, 5)])
    voice = Voice([container_1, Note(2, (1, 8)), container_2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8
                cs'8
            }
            d'8
            {
                ef'8
                e'8
            }
        }
        '''
        )

    assert inspect_(container_1[1]).get_leaf(1) is voice[1]
    assert inspect_(voice[1]).get_leaf(1) is container_2[0]

    assert inspect_(voice[1]).get_leaf(-1) is container_1[1]
    assert inspect_(container_2[0]).get_leaf(-1) is voice[1]


def test_agenttools_InspectionAgent_get_leaf_17():
    r'''Does connect in sequence of alternating tuplets and notes.
    '''

    notes = [Note(i, Duration(1, 8)) for i in range(3)]
    tuplet_1 = scoretools.FixedDurationTuplet(Duration(1, 4), notes)
    notes = [Note(i, Duration(1, 8)) for i in range(4, 7)]
    tuplet_2 = scoretools.FixedDurationTuplet(Duration(1, 4), notes)
    voice = Voice([tuplet_1, Note(3, (1, 8)), tuplet_2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8
                cs'8
                d'8
            }
            ef'8
            \times 2/3 {
                e'8
                f'8
                fs'8
            }
        }
        '''
        )

    assert inspect_(tuplet_1[-1]).get_leaf(1) is voice[1]
    assert inspect_(voice[1]).get_leaf(1) is tuplet_2[0]

    assert inspect_(voice[1]).get_leaf(-1) is tuplet_1[-1]
    assert inspect_(tuplet_2[0]).get_leaf(-1) is voice[1]


def test_agenttools_InspectionAgent_get_leaf_18():
    r'''Does connect through asymmetrically nested tuplets.
    '''

    inner_tuplet = scoretools.FixedDurationTuplet((1, 4), "c'8 c'8 c'8")
    contents = [Note("c'4"), inner_tuplet, Note("c'4")]
    tuplet = scoretools.FixedDurationTuplet(Duration(2, 4), contents)

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'4
            \times 2/3 {
                c'8
                c'8
                c'8
            }
            c'4
        }
        '''
        )

    assert inspect_(tuplet[0]).get_leaf(1) is inner_tuplet[0]
    assert inspect_(inner_tuplet[-1]).get_leaf(1) is tuplet[-1]
    assert inspect_(tuplet[-1]).get_leaf(-1) is inner_tuplet[-1]
    assert inspect_(inner_tuplet[0]).get_leaf(-1) is tuplet[0]


def test_agenttools_InspectionAgent_get_leaf_19():
    r'''Returns none in asymmetric logical voice parentage structures.
    '''

    voice_1 = Voice([Note(i, (1, 8)) for i in range(3)])
    note = Note(3, (1, 8))
    voice_2 = Voice([Note(i, (1, 8)) for i in range(4, 8)])
    staff = Staff([voice_1, note, voice_2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \new Voice {
                c'8
                cs'8
                d'8
            }
            ef'8
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    assert inspect_(voice_1[-1]).get_leaf(1) is None
    assert inspect_(note).get_leaf(1) is None

    assert inspect_(voice_2[0]).get_leaf(-1) is None
    assert inspect_(note).get_leaf(-1) is None


def test_agenttools_InspectionAgent_get_leaf_20():
    r'''Noncontiguous or broken logical voices do not connect.
    '''

    voice_1 = Voice([Note(i, (1, 8)) for i in range(3)])
    voice_1.name = 'My Voice'
    voice_2 = Voice([Note(i, (1, 8)) for i in range(4, 8)])
    voice_2.name = 'Your Voice'
    voice_3 = Voice([Note(i, (1, 8)) for i in range(4, 8)])
    voice_3.name = 'My Voice'
    staff = Staff([voice_1, voice_2, voice_3])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \context Voice = "My Voice" {
                c'8
                cs'8
                d'8
            }
            \context Voice = "Your Voice" {
                e'8
                f'8
                fs'8
                g'8
            }
            \context Voice = "My Voice" {
                e'8
                f'8
                fs'8
                g'8
            }
        }
        '''
        )

    assert inspect_(voice_1[-1]).get_leaf(1) is None
    assert inspect_(voice_2[-1]).get_leaf(1) is None

    voice_2.name = None

    assert inspect_(voice_1[-1]).get_leaf(1) is None
    assert inspect_(voice_2[-1]).get_leaf(1) is None

    assert inspect_(voice_3[0]).get_leaf(-1) is None
    assert inspect_(voice_2[0]).get_leaf(-1) is None


def test_agenttools_InspectionAgent_get_leaf_21():
    r'''Does not connect through nested anonymous voices.
    '''

    inner_voice = Voice([Note(i, (1, 8)) for i in range(3)])
    outer_voice = Voice([inner_voice, Note(3, (1, 8))])

    assert format(outer_voice) == stringtools.normalize(
        r'''
        \new Voice {
            \new Voice {
                c'8
                cs'8
                d'8
            }
            ef'8
        }
        '''
        )

    assert inspect_(inner_voice[0]).get_leaf(1) is inner_voice[1]
    assert inspect_(inner_voice[1]).get_leaf(1) is inner_voice[2]
    assert inspect_(inner_voice[2]).get_leaf(1) is None

    assert inspect_(inner_voice[1]).get_leaf(-1) is inner_voice[0]
    assert inspect_(inner_voice[2]).get_leaf(-1) is inner_voice[1]
    assert inspect_(outer_voice[1]).get_leaf(-1) is None


def test_agenttools_InspectionAgent_get_leaf_22():
    r'''Does not connect through nested anonymous voices.
    '''

    inner_voice = Voice([Note(i, (1, 8)) for i in range(1, 4)])
    outer_voice = Voice([Note(0, (1, 8)), inner_voice])

    assert format(outer_voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8
            \new Voice {
                cs'8
                d'8
                ef'8
            }
        }
        '''
        )

    assert inspect_(inner_voice[0]).get_leaf(1) is inner_voice[1]
    assert inspect_(inner_voice[1]).get_leaf(1) is inner_voice[2]
    assert inspect_(outer_voice[0]).get_leaf(1) is None

    assert inspect_(inner_voice[1]).get_leaf(-1) is inner_voice[0]
    assert inspect_(inner_voice[2]).get_leaf(-1) is inner_voice[1]
    assert inspect_(inner_voice[0]).get_leaf(-1) is None



def test_agenttools_InspectionAgent_get_leaf_23():
    r'''Does connect through nested equally named voices.
    '''

    inner_voice = Voice([Note(i, (1, 8)) for i in range(3)])
    inner_voice.name = 'My Voice'
    outer_voice = Voice([inner_voice, Note(3, (1, 8))])
    outer_voice.name = 'My Voice'

    assert format(outer_voice) == stringtools.normalize(
        r'''
        \context Voice = "My Voice" {
            \context Voice = "My Voice" {
                c'8
                cs'8
                d'8
            }
            ef'8
        }
        '''
        )

    assert inspect_(inner_voice[0]).get_leaf(1) is inner_voice[1]
    assert inspect_(inner_voice[1]).get_leaf(1) is inner_voice[2]
    assert inspect_(inner_voice[2]).get_leaf(1) is outer_voice[1]

    assert inspect_(inner_voice[1]).get_leaf(-1) is inner_voice[0]
    assert inspect_(inner_voice[2]).get_leaf(-1) is inner_voice[1]
    assert inspect_(outer_voice[1]).get_leaf(-1) is inner_voice[-1]


def test_agenttools_InspectionAgent_get_leaf_24():
    r'''Does connect through nested equally named voices.
    '''

    inner_voice = Voice([Note(i, (1, 8)) for i in range(1, 4)])
    inner_voice.name = 'My Voice'
    outer_voice = Voice([Note(0, (1, 8)), inner_voice])
    outer_voice.name = 'My Voice'

    assert format(outer_voice) == stringtools.normalize(
        r'''
        \context Voice = "My Voice" {
            c'8
            \context Voice = "My Voice" {
                cs'8
                d'8
                ef'8
            }
        }
        '''
        )

    assert inspect_(inner_voice[0]).get_leaf(1) is inner_voice[1]
    assert inspect_(inner_voice[1]).get_leaf(1) is inner_voice[2]
    assert inspect_(outer_voice[0]).get_leaf(1) is inner_voice[0]

    assert inspect_(inner_voice[1]).get_leaf(-1) is inner_voice[0]
    assert inspect_(inner_voice[2]).get_leaf(-1) is inner_voice[1]
    assert inspect_(inner_voice[0]).get_leaf(-1) is outer_voice[0]


def test_agenttools_InspectionAgent_get_leaf_25():
    r'''Returns none on nested differently named voices.
    '''

    inner_voice = Voice([Note(i, (1, 8)) for i in range(3)])
    inner_voice.name = 'Your Voice'
    outer_voice = Voice([inner_voice, Note(3, (1, 8))])
    outer_voice.name = 'My Voice'

    assert format(outer_voice) == stringtools.normalize(
        r'''
        \context Voice = "My Voice" {
            \context Voice = "Your Voice" {
                c'8
                cs'8
                d'8
            }
            ef'8
        }
        '''
        )

    assert inspect_(inner_voice[0]).get_leaf(1) is inner_voice[1]
    assert inspect_(inner_voice[1]).get_leaf(1) is inner_voice[2]
    assert inspect_(inner_voice[2]).get_leaf(1) is None

    assert inspect_(inner_voice[1]).get_leaf(-1) is inner_voice[0]
    assert inspect_(inner_voice[2]).get_leaf(-1) is inner_voice[1]
    assert inspect_(outer_voice[1]).get_leaf(-1) is None


def test_agenttools_InspectionAgent_get_leaf_26():
    r'''Returns none on nested differently named voices.
    '''

    voice_2 = Voice([Note(i, (1, 8)) for i in range(1, 4)])
    voice_2.name = 'Voice 2'
    voice_1 = Voice([Note(0, (1, 8)), voice_2])
    voice_1.name = 'Voice 1'

    assert format(voice_1) == stringtools.normalize(
        r'''
        \context Voice = "Voice 1" {
            c'8
            \context Voice = "Voice 2" {
                cs'8
                d'8
                ef'8
            }
        }
        '''
        )

    assert inspect_(voice_2[0]).get_leaf(1) is voice_2[1]
    assert inspect_(voice_2[1]).get_leaf(1) is voice_2[2]
    assert inspect_(voice_1[0]).get_leaf(1) is None

    assert inspect_(voice_2[1]).get_leaf(-1) is voice_2[0]
    assert inspect_(voice_2[2]).get_leaf(-1) is voice_2[1]
    assert inspect_(voice_1[1]).get_leaf(-1) is voice_2[-1]



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Tuplet___setattr___01():
    r'''Slots constrain tuplet attributes.
    '''

    tuplet = Tuplet(Multiplier(2, 3), "c'8 d'8 e'8")

    assert pytest.raises(AttributeError, "tuplet.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.timespantools.TimeRelation import TimeRelation


class OffsetTimespanTimeRelation(TimeRelation):
    r'''An offset vs. timespan time relation.

    ::

        >>> offset = Offset(5)
        >>> timespan = timespantools.Timespan(0, 10)
        >>> time_relation = timespantools.offset_happens_during_timespan(
        ...     offset=offset,
        ...     timespan=timespan,
        ...     hold=True,
        ...     )

    ::

        >>> print(format(time_relation))
        timespantools.OffsetTimespanTimeRelation(
            inequality=timespantools.CompoundInequality(
                [
                    timespantools.Inequality('timespan.start <= offset'),
                    timespantools.Inequality('offset < timespan.stop'),
                    ],
                logical_operator='and',
                ),
            timespan=timespantools.Timespan(
                start_offset=durationtools.Offset(0, 1),
                stop_offset=durationtools.Offset(10, 1),
                ),
            offset=durationtools.Offset(5, 1),
            )

    Offset / timespan time relations are immutable.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Time relations'

    __slots__ = (
        '_offset',
        '_timespan',
        )

    ### INITIALIZER ###

    def __init__(self, inequality=None, timespan=None, offset=None):
        TimeRelation.__init__(self, inequality=inequality)
        self._timespan = timespan
        self._offset = offset

    ### SPECIAL METHODS ###

    # TODO: hoist to TimeRelation
    def __call__(self, timespan=None, offset=None):
        r'''Evaluates time relation:

            >>> time_relation()
            True

        Raises value error is either `offset` or `timespan` is none.

        Otherwise returns boolean.
        '''
        from abjad.tools import timespantools
        timespan = timespan or self.timespan
        offset = offset or self.offset
        if timespan is None or offset is None:
            message = 'time relation is not fully loaded.'
            raise ValueError(message)
        if not isinstance(timespan, timespantools.Timespan):
            timespan = timespantools.Timespan()._get_timespan(timespan)
        offset = durationtools.Offset(offset)
        truth_value = self.inequality.evaluate_offset_inequality(
            timespan.start_offset, timespan.stop_offset, offset)
        return truth_value

    def __eq__(self, argument):
        r'''Is true when `argument` equals time relation. Otherwise false:

        ::

            >>> offset = Offset(5)
            >>> time_relation_1 = \
            ...     timespantools.offset_happens_during_timespan()
            >>> time_relation_2 = \
            ...     timespantools.offset_happens_during_timespan(
            ...     offset=offset)

        ::

            >>> time_relation_1 == time_relation_1
            True
            >>> time_relation_1 == time_relation_2
            False
            >>> time_relation_2 == time_relation_2
            True

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            if self.inequality == argument.inequality:
                if self.timespan == argument.timespan:
                    if self.offset == argument.offset:
                        return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats time relation.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ::

            >>> print(format(time_relation))
            timespantools.OffsetTimespanTimeRelation(
                inequality=timespantools.CompoundInequality(
                    [
                        timespantools.Inequality('timespan.start <= offset'),
                        timespantools.Inequality('offset < timespan.stop'),
                        ],
                    logical_operator='and',
                    ),
                timespan=timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(10, 1),
                    ),
                offset=durationtools.Offset(5, 1),
                )

        Returns string.
        '''
        superclass = super(OffsetTimespanTimeRelation, self)
        return superclass.__format__(format_specification=format_specification)

    def __hash__(self):
        r'''Hashes time relation.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(OffsetTimespanTimeRelation, self).__hash__()

    ### PUBLIC PROPERTIES ###

    @property
    def is_fully_loaded(self):
        r'''Is true when `timespan` and `offset` are both not none.
        Otherwise false:

        ::

            >>> time_relation.is_fully_loaded
            True

        Returns true or false.
        '''
        return self.timespan is not None and self.offset is not None

    @property
    def is_fully_unloaded(self):
        r'''Is true when `timespan` and `offset` are both none.
        Otherwise false:

        ::

            >>> time_relation.is_fully_unloaded
            False

        Returns true or false.
        '''
        return self.timespan is None and self.offset is None

    @property
    def offset(self):
        r'''Time relation offset:

        ::

            >>> time_relation.offset
            Offset(5, 1)

        Returns offset or none.
        '''
        return self._offset

    @property
    def timespan(self):
        r'''Time relation timepsan:

        ::

            >>> time_relation.timespan
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(10, 1))

        Returns timespan or none.
        '''
        return self._timespan



# -*- coding: utf-8 -*-
import platform
import unittest
from abjad.tools import abjadbooktools


@unittest.skipIf(
    platform.python_implementation() != 'CPython',
    'Only for CPython',
    )
class TestLaTeXDocumentHandler_invalid(unittest.TestCase):

    maxDiff = None

    def test_invalid_source_1(self):
        input_file_contents = [
            '<abjad>',
            '<abjad>',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            ValueError,
            document_handler.__call__,
            )

    def test_invalid_source_2(self):
        input_file_contents = [
            '<abjad>',
            '%%% ABJADBOOK START %%%',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            ValueError,
            document_handler.__call__,
            )

    def test_invalid_source_3(self):
        input_file_contents = [
            '<abjad>',
            '%%% ABJADBOOK END %%%',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            ValueError,
            document_handler.__call__,
            )

    def test_invalid_source_4(self):
        input_file_contents = [
            '%%% ABJADBOOK START %%%',
            '%%% ABJADBOOK START %%%',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            ValueError,
            document_handler.__call__,
            )

    def test_invalid_source_5(self):
        input_file_contents = [
            '%%% ABJADBOOK START %%%',
            '<abjad>',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            ValueError,
            document_handler.__call__,
            )

    def test_invalid_source_6(self):
        input_file_contents = [
            '%%% ABJADBOOK START %%%',
            '</abjad>',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            ValueError,
            document_handler.__call__,
            )

    def test_invalid_source_7(self):
        input_file_contents = [
            ''
            '</abjad>',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            ValueError,
            document_handler.__call__,
            )

    def test_invalid_source_8(self):
        input_file_contents = [
            ''
            '%%% ABJADBOOK END %%%',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            ValueError,
            document_handler.__call__,
            )



# -*- coding: utf-8 -*-
import os
from abjad import *
configuration = systemtools.AbjadConfiguration()

path_1 = os.path.join(
    configuration.abjad_directory,
    'test_1.ly',
    )
path_2 = os.path.join(
    configuration.abjad_directory,
    'test_2.ly',
    )

lines = [
    r'\language "english"',
    '',
    r'\new Staff {',
    "    c'4",
    "    d'4",
    "    e'4",
    "    f'4",
    ]


def test_systemtools_TestManager__compare_lys_01():
    r'''Is true when lines are exactly the same.
    '''

    with systemtools.FilesystemState(remove=[path_1, path_2]):
        first_lines = [r'\version "2.19.7"'] + lines
        second_lines = first_lines[:]
        with open(path_1, 'w') as file_pointer:
            file_pointer.write('\n'.join(first_lines))
        with open(path_2, 'w') as file_pointer:
            file_pointer.write('\n'.join(second_lines))
        assert systemtools.TestManager._compare_lys(path_1, path_2)


def test_systemtools_TestManager__compare_lys_02():
    r'''Is true when version strings differ.
    '''

    with systemtools.FilesystemState(remove=[path_1, path_2]):
        first_lines = [r'\version "2.19.7"'] + lines
        second_lines = [r'\version "2.19.8"'] + lines
        with open(path_1, 'w') as file_pointer:
            file_pointer.write('\n'.join(first_lines))
        with open(path_2, 'w') as file_pointer:
            file_pointer.write('\n'.join(second_lines))
        assert systemtools.TestManager._compare_lys(path_1, path_2)


def test_systemtools_TestManager__compare_lys_03():
    r'''Is true when comments differ.
    '''

    with systemtools.FilesystemState(remove=[path_1, path_2]):
        first_lines = [r'\version "2.19.7"'] + lines
        second_lines = ['% 2014-01-01 05:43:01', r'\version "2.19.8"'] + lines
        with open(path_1, 'w') as file_pointer:
            file_pointer.write('\n'.join(first_lines))
        with open(path_2, 'w') as file_pointer:
            file_pointer.write('\n'.join(second_lines))
        assert systemtools.TestManager._compare_lys(path_1, path_2)


def test_systemtools_TestManager__compare_lys_04():
    r'''False when any other lines differ.
    '''

    with systemtools.FilesystemState(remove=[path_1, path_2]):
        first_lines = [r'\version "2.19.7"'] + lines
        second_lines = [r'\version "2.19.8"'] + lines + ['foo']
        with open(path_1, 'w') as file_pointer:
            file_pointer.write('\n'.join(first_lines))
        with open(path_2, 'w') as file_pointer:
            file_pointer.write('\n'.join(second_lines))
        assert not systemtools.TestManager._compare_lys(path_1, path_2)



# -*- coding: utf-8 -*-


def difference_series(argument):
    r'''Gets difference series of `argument`.

    ..  container:: example

        ::

            >>> mathtools.difference_series([1, 1, 2, 3, 5, 5, 6])
            [0, 1, 1, 2, 0, 1]

        ::

            >>> mathtools.difference_series([9, 6, 8, 5, 7, 4, 6])
            [-3, 2, -3, 2, -3, 2]

    Returns new object of `argument` type.
    '''
    result = []
    for i, n in enumerate(argument[1:]):
        result.append(n - argument[i])
    return type(argument)(result)



# -*- coding: utf-8 -*-
from abjad.tools.schemetools.Scheme import Scheme


class SchemeColor(Scheme):
    r'''A Scheme color.

    ..  container:: example

        ::

            >>> schemetools.SchemeColor('ForestGreen')
            SchemeColor('ForestGreen')

    Scheme colors are immutable.
    '''

    ### CLASS VARIABLES ##

    __slots__ = (
        )

    ### PRIVATE PROPERTIES ###

    @property
    def _formatted_value(self):
        string = "(x11-color '{})"
        string = string.format(self._value)
        return string



# -*- coding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools import systemtools
from abjad.tools.schemetools.Scheme import Scheme


class SchemeVector(Scheme):
    '''Abjad model of Scheme vector.

    ..  container:: example

        **Example 1.** Scheme vector of boolean values:

        ::

            >>> scheme = schemetools.SchemeVector(True, True, False)
            >>> scheme
            SchemeVector(True, True, False)
            >>> print(format(scheme))
            #'(#t #t #f)

    ..  container:: example

        **Example 2.** Scheme vector of symbols:

        ::

            >>> scheme = schemetools.SchemeVector('foo', 'bar', 'blah')
            >>> scheme
            SchemeVector('foo', 'bar', 'blah')
            >>> print(format(scheme))
            #'(foo bar blah)

    Scheme vectors and Scheme vector constants differ in only
    their LilyPond input format.

    Scheme vectors are immutable.
    '''

    ### CLASS VARIABLES ##

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, *arguments):
        Scheme.__init__(self, *arguments, quoting="'")

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = self._value
        if stringtools.is_string(self._value):
            values = [self._value]
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=values,
            storage_format_kwargs_names=[],
            )



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools.TreeNode import TreeNode


class ReSTAutosummaryItem(TreeNode):
    r'''A ReST autosummary item.

    ::

        >>> item = documentationtools.ReSTAutosummaryItem(
        ...     text='abjad.tools.scoretools.Note')
        >>> item
        ReSTAutosummaryItem(
            text='abjad.tools.scoretools.Note'
            )

    ::

        >>> print(item.rest_format)
        abjad.tools.scoretools.Note

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### INITIALIZER ###

    def __init__(self, name=None, text='foo'):
        TreeNode.__init__(self, name)
        self.text = text

    ### PRIVATE PROPERTIES ###

    @property
    def _rest_format_contributions(self):
        return [self.text]

    ### PUBLIC PROPERTIES ###

    @property
    def rest_format(self):
        r'''ReST format of ReST autosummary item.
        '''
        return '\n'.join(self._rest_format_contributions)

    @property
    def text(self):
        r'''Text of ReST autosummary item.
        '''
        return self._text

    @text.setter
    def text(self, arg):
        assert isinstance(arg, str)
        arg = arg.strip()
        assert len(arg)
        self._text = arg



# -*- coding: utf-8 -*-
import collections
from abjad.tools import mathtools
from abjad.tools.abctools import AbjadValueObject


class Postscript(AbjadValueObject):
    r'''Postscript session.

    ..  note::

        The markup resulting from the ``\postscript`` markup command is both
        0-height and 0-width. Make sure to wrap the ``\postscript`` command
        with a ``\pad-to-box`` or ``\with-dimensions`` markup command to give
        it explicit height and width. Likewise, use only positive coordinates
        in your postscript markup if at all possible. When specifying explicit
        extents with ``\pad-to-box`` or ``\with-dimensions``, negative extents
        will *not* be interpreted by LilyPond as resulting in positive height
        or width, and may have unexpected behavior.

    ..  note::

        LilyPond will fail to render if *any* of the font commands are used. To
        create text, use ``.show('text')`` preceded by ``.scale()`` or
        ``.rotate()`` to provide the appropriate transformation.
        ``.charpath()`` is also useable. However, ``.findfont()``,
        ``.scalefont()``, ``.setfont()`` will cause LilyPond to error.

    ..  container:: example

        ::

            >>> postscript = markuptools.Postscript()
            >>> postscript = postscript.moveto(1, 1)
            >>> postscript = postscript.setlinewidth(2.5)
            >>> postscript = postscript.setdash((2, 1))
            >>> postscript = postscript.lineto(3, -4)
            >>> postscript = postscript.stroke()
            >>> print(format(postscript))
            markuptools.Postscript(
                operators=(
                    markuptools.PostscriptOperator('moveto', 1.0, 1.0),
                    markuptools.PostscriptOperator('setlinewidth', 2.5),
                    markuptools.PostscriptOperator('setdash', (2.0, 1.0), 0.0),
                    markuptools.PostscriptOperator('lineto', 3.0, -4.0),
                    markuptools.PostscriptOperator('stroke'),
                    ),
                )

        ::

            >>> print(str(postscript))
            1 1 moveto
            2.5 setlinewidth
            [ 2 1 ] 0 setdash
            3 -4 lineto
            stroke

        ::

            >>> postscript = markuptools.Postscript()
            >>> postscript = postscript.newpath()
            >>> postscript = postscript.moveto(0, 0)
            >>> postscript = postscript.rlineto(0, -10)
            >>> postscript = postscript.rlineto(10, 0)
            >>> postscript = postscript.rlineto(0, 10)
            >>> postscript = postscript.rlineto(-10, 0)
            >>> postscript = postscript.closepath()
            >>> postscript = postscript.gsave()
            >>> postscript = postscript.setrgbcolor(0.5, 1, 0.5)
            >>> postscript = postscript.fill()
            >>> postscript = postscript.grestore()
            >>> postscript = postscript.setrgbcolor(1, 0, 0)
            >>> postscript = postscript.setlinewidth(1)
            >>> postscript = postscript.stroke()
            >>> show(postscript) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_operators',
        )

    ### INITIALIZER ###

    def __init__(self, operators=None):
        from abjad.tools import markuptools
        prototype = markuptools.PostscriptOperator
        if operators is not None:
            assert all(isinstance(_, prototype) for _ in operators)
            operators = tuple(operators)
        self._operators = operators

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        r'''Adds postscript to `argument`.

        Returns new postscript.
        '''
        assert isinstance(argument, type(self))
        self_operators = self.operators or ()
        expr_operators = argument.operators or ()
        operators = self_operators + expr_operators
        operators = operators or None
        return type(self)(operators)

    def __illustrate__(self):
        r'''Illustrates Postscript.

        Returns LilyPond file.
        '''
        import abjad
        markup = abjad.Markup.postscript(self)
        return markup.__illustrate__()

    def __str__(self):
        r'''Gets string representation of Postscript.

        Return string.
        '''
        if not self.operators:
            return ''
        return '\n'.join(str(_) for _ in self.operators)

    ### PRIVATE METHODS ###

    @staticmethod
    def _format_argument(argument):
        if isinstance(argument, str):
            if argument.startswith('/'):
                return argument
            return '({})'.format(argument)
        elif isinstance(argument, collections.Sequence):
            if not argument:
                return '[ ]'
            contents = ' '.join(
                Postscript._format_argument(_) for _ in argument
                )
            return '[ {} ]'.format(contents)
        elif isinstance(argument, bool):
            return str(argument).lower()
        elif isinstance(argument, (int, float)):
            argument = mathtools.integer_equivalent_number_to_integer(argument)
            return str(argument)
        return str(argument)

    def _with_operator(self, operator):
        operators = self.operators or ()
        operators = operators + (operator,)
        return type(self)(operators)

    ### PUBLIC METHODS ###

    def as_markup(self):
        r'''Converts postscript to markup.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 200)
                >>> postscript = postscript.lineto(200, 250)
                >>> postscript = postscript.lineto(100, 300)
                >>> postscript = postscript.closepath()
                >>> postscript = postscript.gsave()
                >>> postscript = postscript.setgray(0.5)
                >>> postscript = postscript.fill()
                >>> postscript = postscript.grestore()
                >>> postscript = postscript.setlinewidth(4)
                >>> postscript = postscript.setgray(0.75)
                >>> postscript = postscript.stroke()

            ::

                >>> markup = postscript.as_markup()
                >>> print(format(markup))
                \markup {
                    \postscript
                        #"
                        newpath
                        100 200 moveto
                        200 250 lineto
                        100 300 lineto
                        closepath
                        gsave
                        0.5 setgray
                        fill
                        grestore
                        4 setlinewidth
                        0.75 setgray
                        stroke
                        "
                    }

        Returns new markup.
        '''
        from abjad.tools import markuptools
        return markuptools.Markup.postscript(self)

    def charpath(self, text, modify_font=True):
        r'''Postscript ``charpath`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.findfont('Times Roman')
                >>> postscript = postscript.scalefont(32)
                >>> postscript = postscript.setfont()
                >>> postscript = postscript.translate(100, 200)
                >>> postscript = postscript.rotate(45)
                >>> postscript = postscript.scale(2, 1)
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(0, 0)
                >>> postscript = postscript.charpath('This is text.', True)
                >>> postscript = postscript.setlinewidth(0.5)
                >>> postscript = postscript.setgray(0.25)
                >>> postscript = postscript.stroke()
                >>> print(str(postscript))
                /Times-Roman findfont
                32 scalefont
                setfont
                100 200 translate
                45 rotate
                2 1 scale
                newpath
                0 0 moveto
                (This is text.) true charpath
                0.5 setlinewidth
                0.25 setgray
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        text = str(text)
        modify_font = bool(modify_font)
        operator = markuptools.PostscriptOperator(
            'charpath',
            text,
            modify_font,
            )
        return self._with_operator(operator)

    def closepath(self):
        r'''Postscript ``closepath`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 200)
                >>> postscript = postscript.lineto(200, 250)
                >>> postscript = postscript.lineto(100, 300)
                >>> postscript = postscript.closepath()
                >>> postscript = postscript.gsave()
                >>> postscript = postscript.setgray(0.5)
                >>> postscript = postscript.fill()
                >>> postscript = postscript.grestore()
                >>> postscript = postscript.setlinewidth(4)
                >>> postscript = postscript.setgray(0.75)
                >>> postscript = postscript.stroke()
                >>> print(str(postscript))
                newpath
                100 200 moveto
                200 250 lineto
                100 300 lineto
                closepath
                gsave
                0.5 setgray
                fill
                grestore
                4 setlinewidth
                0.75 setgray
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        operator = markuptools.PostscriptOperator('closepath')
        return self._with_operator(operator)

    def curveto(self, x1, y1, x2, y2, x3, y3):
        r'''Postscript ``curveto`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.curveto(0, 1, 1.5, 2, 3, 6)
                >>> print(str(postscript))
                0 1 1.5 2 3 6 curveto

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        x1 = float(x1)
        x2 = float(x2)
        x3 = float(x3)
        y1 = float(y1)
        y2 = float(y2)
        y3 = float(y3)
        operator = markuptools.PostscriptOperator(
            'curveto',
            x1, y1,
            x2, y2,
            x3, y3,
            )
        return self._with_operator(operator)

    def fill(self):
        r'''Postscript ``fill`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 200)
                >>> postscript = postscript.lineto(200, 250)
                >>> postscript = postscript.lineto(100, 300)
                >>> postscript = postscript.closepath()
                >>> postscript = postscript.gsave()
                >>> postscript = postscript.setgray(0.5)
                >>> postscript = postscript.fill()
                >>> postscript = postscript.grestore()
                >>> postscript = postscript.setlinewidth(4)
                >>> postscript = postscript.setgray(0.75)
                >>> postscript = postscript.stroke()
                >>> print(str(postscript))
                newpath
                100 200 moveto
                200 250 lineto
                100 300 lineto
                closepath
                gsave
                0.5 setgray
                fill
                grestore
                4 setlinewidth
                0.75 setgray
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        operator = markuptools.PostscriptOperator('fill')
        return self._with_operator(operator)

    def findfont(self, font_name):
        r'''Postscript ``findfont`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.findfont('Times Roman')
                >>> postscript = postscript.scalefont(12)
                >>> postscript = postscript.setfont()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 200)
                >>> postscript = postscript.show('This is text.')
                >>> print(str(postscript))
                /Times-Roman findfont
                12 scalefont
                setfont
                newpath
                100 200 moveto
                (This is text.) show

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        font_name = str(font_name)
        font_name = font_name.replace(' ', '-')
        font_name = '/{}'.format(font_name)
        operator = markuptools.PostscriptOperator('findfont', font_name)
        return self._with_operator(operator)

    def grestore(self):
        r'''Postscript ``grestore`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 200)
                >>> postscript = postscript.lineto(200, 250)
                >>> postscript = postscript.lineto(100, 300)
                >>> postscript = postscript.closepath()
                >>> postscript = postscript.gsave()
                >>> postscript = postscript.setgray(0.5)
                >>> postscript = postscript.fill()
                >>> postscript = postscript.grestore()
                >>> postscript = postscript.setlinewidth(4)
                >>> postscript = postscript.setgray(0.75)
                >>> postscript = postscript.stroke()
                >>> print(str(postscript))
                newpath
                100 200 moveto
                200 250 lineto
                100 300 lineto
                closepath
                gsave
                0.5 setgray
                fill
                grestore
                4 setlinewidth
                0.75 setgray
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        operator = markuptools.PostscriptOperator('grestore')
        return self._with_operator(operator)

    def gsave(self):
        r'''Postscript ``gsave`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 200)
                >>> postscript = postscript.lineto(200, 250)
                >>> postscript = postscript.lineto(100, 300)
                >>> postscript = postscript.closepath()
                >>> postscript = postscript.gsave()
                >>> postscript = postscript.setgray(0.5)
                >>> postscript = postscript.fill()
                >>> postscript = postscript.grestore()
                >>> postscript = postscript.setlinewidth(4)
                >>> postscript = postscript.setgray(0.75)
                >>> postscript = postscript.stroke()
                >>> print(str(postscript))
                newpath
                100 200 moveto
                200 250 lineto
                100 300 lineto
                closepath
                gsave
                0.5 setgray
                fill
                grestore
                4 setlinewidth
                0.75 setgray
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        operator = markuptools.PostscriptOperator('gsave')
        return self._with_operator(operator)

    def lineto(self, x, y):
        r'''Postscript ``lineto`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.moveto(1, 1)
                >>> postscript = postscript.lineto(3, -4)
                >>> postscript = postscript.stroke()
                >>> print(str(postscript))
                1 1 moveto
                3 -4 lineto
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        x = float(x)
        y = float(y)
        operator = markuptools.PostscriptOperator('lineto', x, y)
        return self._with_operator(operator)

    def moveto(self, x, y):
        r'''Postscript ``moveto`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.moveto(1, 1)
                >>> postscript = postscript.lineto(3, -4)
                >>> postscript = postscript.stroke()
                >>> print(format(postscript))
                markuptools.Postscript(
                    operators=(
                        markuptools.PostscriptOperator('moveto', 1.0, 1.0),
                        markuptools.PostscriptOperator('lineto', 3.0, -4.0),
                        markuptools.PostscriptOperator('stroke'),
                        ),
                    )

            ::

                >>> print(str(postscript))
                1 1 moveto
                3 -4 lineto
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        x = float(x)
        y = float(y)
        operator = markuptools.PostscriptOperator('moveto', x, y)
        return self._with_operator(operator)

    def newpath(self):
        r'''Postscript ``newpath`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 200)
                >>> postscript = postscript.lineto(200, 250)
                >>> postscript = postscript.lineto(100, 300)
                >>> postscript = postscript.closepath()
                >>> postscript = postscript.gsave()
                >>> postscript = postscript.setgray(0.5)
                >>> postscript = postscript.fill()
                >>> postscript = postscript.grestore()
                >>> postscript = postscript.setlinewidth(4)
                >>> postscript = postscript.setgray(0.75)
                >>> postscript = postscript.stroke()
                >>> print(str(postscript))
                newpath
                100 200 moveto
                200 250 lineto
                100 300 lineto
                closepath
                gsave
                0.5 setgray
                fill
                grestore
                4 setlinewidth
                0.75 setgray
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        operator = markuptools.PostscriptOperator('newpath')
        return self._with_operator(operator)

    def rcurveto(self, dx1, dy1, dx2, dy2, dx3, dy3):
        r'''Postscript ``rcurveto`` operator.

        ..  container:: edxample

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.rcurveto(0, 1, 1.5, 2, 3, 6)
                >>> print(str(postscript))
                0 1 1.5 2 3 6 rcurveto

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        dx1 = float(dx1)
        dx2 = float(dx2)
        dx3 = float(dx3)
        dy1 = float(dy1)
        dy2 = float(dy2)
        dy3 = float(dy3)
        operator = markuptools.PostscriptOperator(
            'rcurveto',
            dx1, dy1,
            dx2, dy2,
            dx3, dy3,
            )
        return self._with_operator(operator)

    def rlineto(self, dx, dy):
        r'''Postscript ``rlineto`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.rmoveto(1, 1)
                >>> postscript = postscript.rlineto(3, -4)
                >>> postscript = postscript.stroke()
                >>> print(format(postscript))
                markuptools.Postscript(
                    operators=(
                        markuptools.PostscriptOperator('rmoveto', 1.0, 1.0),
                        markuptools.PostscriptOperator('rlineto', 3.0, -4.0),
                        markuptools.PostscriptOperator('stroke'),
                        ),
                    )

            ::

                >>> print(str(postscript))
                1 1 rmoveto
                3 -4 rlineto
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        dx = float(dx)
        dy = float(dy)
        operator = markuptools.PostscriptOperator('rlineto', dx, dy)
        return self._with_operator(operator)

    def rmoveto(self, dx, dy):
        r'''Postscript ``rmoveto`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.rmoveto(1, 1)
                >>> postscript = postscript.rlineto(3, -4)
                >>> postscript = postscript.stroke()
                >>> print(format(postscript))
                markuptools.Postscript(
                    operators=(
                        markuptools.PostscriptOperator('rmoveto', 1.0, 1.0),
                        markuptools.PostscriptOperator('rlineto', 3.0, -4.0),
                        markuptools.PostscriptOperator('stroke'),
                        ),
                    )

            ::

                >>> print(str(postscript))
                1 1 rmoveto
                3 -4 rlineto
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        dx = float(dx)
        dy = float(dy)
        operator = markuptools.PostscriptOperator('rmoveto', dx, dy)
        return self._with_operator(operator)

    def rotate(self, degrees):
        r'''Postscript ``restore`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.findfont('Times Roman')
                >>> postscript = postscript.scalefont(32)
                >>> postscript = postscript.setfont()
                >>> postscript = postscript.translate(100, 200)
                >>> postscript = postscript.rotate(45)
                >>> postscript = postscript.scale(2, 1)
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(0, 0)
                >>> postscript = postscript.charpath('This is text.', True)
                >>> postscript = postscript.setlinewidth(0.5)
                >>> postscript = postscript.setgray(0.25)
                >>> postscript = postscript.stroke()
                >>> print(str(postscript))
                /Times-Roman findfont
                32 scalefont
                setfont
                100 200 translate
                45 rotate
                2 1 scale
                newpath
                0 0 moveto
                (This is text.) true charpath
                0.5 setlinewidth
                0.25 setgray
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        degrees = float(degrees)
        operator = markuptools.PostscriptOperator('rotate', degrees)
        return self._with_operator(operator)

    def scale(self, dx, dy):
        r'''Postscript ``scale`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.findfont('Times Roman')
                >>> postscript = postscript.scalefont(32)
                >>> postscript = postscript.setfont()
                >>> postscript = postscript.translate(100, 200)
                >>> postscript = postscript.rotate(45)
                >>> postscript = postscript.scale(2, 1)
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(0, 0)
                >>> postscript = postscript.charpath('This is text.', True)
                >>> postscript = postscript.setlinewidth(0.5)
                >>> postscript = postscript.setgray(0.25)
                >>> postscript = postscript.stroke()
                >>> print(str(postscript))
                /Times-Roman findfont
                32 scalefont
                setfont
                100 200 translate
                45 rotate
                2 1 scale
                newpath
                0 0 moveto
                (This is text.) true charpath
                0.5 setlinewidth
                0.25 setgray
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        dx = float(dx)
        dy = float(dy)
        operator = markuptools.PostscriptOperator('scale', dx, dy)
        return self._with_operator(operator)

    def scalefont(self, font_size):
        r'''Postscript ``scalefont`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.findfont('Times Roman')
                >>> postscript = postscript.scalefont(12)
                >>> postscript = postscript.setfont()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 200)
                >>> postscript = postscript.show('This is text.')
                >>> print(str(postscript))
                /Times-Roman findfont
                12 scalefont
                setfont
                newpath
                100 200 moveto
                (This is text.) show

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        font_size = float(font_size)
        operator = markuptools.PostscriptOperator('scalefont', font_size)
        return self._with_operator(operator)

    def setdash(self, array=None, offset=0):
        r'''Postscript ``setdash`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript().setdash([2, 1], 3)
                >>> print(format(postscript))
                markuptools.Postscript(
                    operators=(
                        markuptools.PostscriptOperator('setdash', (2.0, 1.0), 3.0),
                        ),
                    )

            ::

                >>> print(str(postscript))
                [ 2 1 ] 3 setdash

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript().setdash()
                >>> print(format(postscript))
                markuptools.Postscript(
                    operators=(
                        markuptools.PostscriptOperator('setdash', (), 0.0),
                        ),
                    )

            ::

                >>> print(str(postscript))
                [ ] 0 setdash

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        if array is None:
            array = ()
        else:
            array = tuple(float(_) for _ in array)
        offset = float(offset)
        operator = markuptools.PostscriptOperator('setdash', array, offset)
        return self._with_operator(operator)

    def setfont(self):
        r'''Postscript ``setfont`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.findfont('Times Roman')
                >>> postscript = postscript.scalefont(12)
                >>> postscript = postscript.setfont()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 200)
                >>> postscript = postscript.show('This is text.')
                >>> print(str(postscript))
                /Times-Roman findfont
                12 scalefont
                setfont
                newpath
                100 200 moveto
                (This is text.) show

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        operator = markuptools.PostscriptOperator('setfont')
        return self._with_operator(operator)

    def setgray(self, gray_value):
        r'''Postscript ``setgray`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 200)
                >>> postscript = postscript.lineto(200, 250)
                >>> postscript = postscript.lineto(100, 300)
                >>> postscript = postscript.closepath()
                >>> postscript = postscript.gsave()
                >>> postscript = postscript.setgray(0.5)
                >>> postscript = postscript.fill()
                >>> postscript = postscript.grestore()
                >>> postscript = postscript.setlinewidth(4)
                >>> postscript = postscript.setgray(0.75)
                >>> postscript = postscript.stroke()
                >>> print(str(postscript))
                newpath
                100 200 moveto
                200 250 lineto
                100 300 lineto
                closepath
                gsave
                0.5 setgray
                fill
                grestore
                4 setlinewidth
                0.75 setgray
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        gray_value = float(gray_value)
        assert 0 <= gray_value <= 1
        operator = markuptools.PostscriptOperator('setgray', gray_value)
        return self._with_operator(operator)

    def setlinewidth(self, width):
        r'''Postscript ``setlinewidth`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.moveto(1, 1)
                >>> postscript = postscript.setlinewidth(2.5)
                >>> postscript = postscript.lineto(3, -4)
                >>> postscript = postscript.stroke()
                >>> print(format(postscript))
                markuptools.Postscript(
                    operators=(
                        markuptools.PostscriptOperator('moveto', 1.0, 1.0),
                        markuptools.PostscriptOperator('setlinewidth', 2.5),
                        markuptools.PostscriptOperator('lineto', 3.0, -4.0),
                        markuptools.PostscriptOperator('stroke'),
                        ),
                    )

            ::

                >>> print(str(postscript))
                1 1 moveto
                2.5 setlinewidth
                3 -4 lineto
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        width = float(width)
        operator = markuptools.PostscriptOperator('setlinewidth', width)
        return self._with_operator(operator)

    def setrgbcolor(self, red, green, blue):
        r'''Postscript ``setrgb`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 100)
                >>> postscript = postscript.rlineto(0, 100)
                >>> postscript = postscript.rlineto(100, 0)
                >>> postscript = postscript.rlineto(0, -100)
                >>> postscript = postscript.rlineto(-100, 0)
                >>> postscript = postscript.closepath()
                >>> postscript = postscript.gsave()
                >>> postscript = postscript.setrgbcolor(0.5, 1, 0.5)
                >>> postscript = postscript.fill()
                >>> postscript = postscript.grestore()
                >>> postscript = postscript.setrgbcolor(1, 0, 0)
                >>> postscript = postscript.setlinewidth(4)
                >>> postscript = postscript.stroke()

            ::

                >>> print(str(postscript))
                newpath
                100 100 moveto
                0 100 rlineto
                100 0 rlineto
                0 -100 rlineto
                -100 0 rlineto
                closepath
                gsave
                0.5 1 0.5 setrgbcolor
                fill
                grestore
                1 0 0 setrgbcolor
                4 setlinewidth
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        red = float(red)
        green = float(green)
        blue = float(blue)
        assert 0 <= red <= 1
        assert 0 <= green <= 1
        assert 0 <= blue <= 1
        operator = markuptools.PostscriptOperator(
            'setrgbcolor',
            red,
            green,
            blue,
            )
        return self._with_operator(operator)

    def show(self, text):
        r'''Postscript ``show`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.findfont('Times Roman')
                >>> postscript = postscript.scalefont(12)
                >>> postscript = postscript.setfont()
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(100, 200)
                >>> postscript = postscript.show('This is text.')
                >>> print(str(postscript))
                /Times-Roman findfont
                12 scalefont
                setfont
                newpath
                100 200 moveto
                (This is text.) show

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        text = str(text)
        operator = markuptools.PostscriptOperator('show', text)
        return self._with_operator(operator)

    def stroke(self):
        r'''Postscript ``stroke`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.lineto(3, -4)
                >>> postscript = postscript.stroke()
                >>> print(format(postscript))
                markuptools.Postscript(
                    operators=(
                        markuptools.PostscriptOperator('lineto', 3.0, -4.0),
                        markuptools.PostscriptOperator('stroke'),
                        ),
                    )

            ::

                >>> print(str(postscript))
                3 -4 lineto
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        operator = markuptools.PostscriptOperator('stroke')
        return self._with_operator(operator)

    def translate(self, dx, dy):
        r'''Postscript ``translate`` operator.

        ..  container:: example

            ::

                >>> postscript = markuptools.Postscript()
                >>> postscript = postscript.findfont('Times Roman')
                >>> postscript = postscript.scalefont(32)
                >>> postscript = postscript.setfont()
                >>> postscript = postscript.translate(100, 200)
                >>> postscript = postscript.rotate(45)
                >>> postscript = postscript.scale(2, 1)
                >>> postscript = postscript.newpath()
                >>> postscript = postscript.moveto(0, 0)
                >>> postscript = postscript.charpath('This is text.', True)
                >>> postscript = postscript.setlinewidth(0.5)
                >>> postscript = postscript.setgray(0.25)
                >>> postscript = postscript.stroke()
                >>> print(str(postscript))
                /Times-Roman findfont
                32 scalefont
                setfont
                100 200 translate
                45 rotate
                2 1 scale
                newpath
                0 0 moveto
                (This is text.) true charpath
                0.5 setlinewidth
                0.25 setgray
                stroke

        Returns new Postscript.
        '''
        from abjad.tools import markuptools
        dx = float(dx)
        dy = float(dy)
        operator = markuptools.PostscriptOperator('translate', dx, dy)
        return self._with_operator(operator)

    ### PUBLIC PROPERTIES ###

    @property
    def operators(self):
        r'''Gets Postscript operators.

        Returns tuple or none.
        '''
        return self._operators



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Container_extend_01():
    r'''Extend container with list of leaves.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])
    voice.extend([Note("c'8"), Note("d'8")])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            c'8
            d'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container_extend_02():
    r'''Extend container with contents of other container.
    '''

    voice_1 = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice_1[:])

    voice_2 = Voice("e'8 f'8")
    beam = Beam()
    attach(beam, voice_2[:])
    voice_1.extend(voice_2)

    assert format(voice_1) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            e'8 [
            f'8 ]
        }
        '''
        )

    assert inspect_(voice_1).is_well_formed()


def test_scoretools_Container_extend_03():
    r'''Extending container with empty list leaves container unchanged.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])
    voice.extend([])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container_extend_04():
    r'''Extending one container with empty second container leaves both
    containers unchanged.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])
    voice.extend(Voice([]))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container_extend_05():
    r'''Trying to extend container with noncomponent raises TypeError.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])

    assert pytest.raises(Exception, 'voice.extend(7)')
    assert pytest.raises(Exception, "voice.extend('foo')")


def test_scoretools_Container_extend_06():
    r'''Trying to extend container with noncontainer raises exception.
    '''

    voice = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice[:])

    statement = 'voice.extend(Note(4, (1, 4)))'
    assert pytest.raises(AttributeError, statement)

    statement = 'voice.extend(Chord([2, 3, 5], (1, 4)))'
    assert pytest.raises(AttributeError, statement)


def test_scoretools_Container_extend_07():
    r'''Extend container with partial and spanned contents of other container.
    '''

    voice_1 = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice_1[:])

    voice_2 = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice_2[:])

    voice_1.extend(voice_2[-2:])

    assert format(voice_1) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            e'8
            f'8
        }
        '''
        )

    assert inspect_(voice_1).is_well_formed()

    assert format(voice_2) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
        }
        '''
        )

    assert inspect_(voice_2).is_well_formed()


def test_scoretools_Container_extend_08():
    r'''Extend container with partial and spanned contents of other container.
    Covered span comes with components from donor container.
    '''

    voice_1 = Voice("c'8 d'8")
    beam = Beam()
    attach(beam, voice_1[:])

    voice_2 = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice_2[:])
    slur = Slur()
    attach(slur, voice_2[-2:])

    assert format(voice_2) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8 (
            f'8 ] )
        }
        '''
        )

    voice_1.extend(voice_2[-2:])

    assert format(voice_1) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            e'8 (
            f'8 )
        }
        '''
        )

    assert inspect_(voice_1).is_well_formed()

    assert format(voice_2) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
        }
        '''
        )

    assert inspect_(voice_2).is_well_formed()


def test_scoretools_Container_extend_09():
    r'''Extend container with LilyPond input string.
    '''

    container = Container([])
    container.extend("c'4 ( d'4 e'4 f'4 )")

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'4 (
            d'4
            e'4
            f'4 )
        }
        '''
        )

    assert inspect_(container).is_well_formed()


def test_scoretools_Container_extend_10():
    r'''Selections are stripped out.
    '''

    selection_1 = scoretools.make_notes([0, 2], [Duration(1, 4)])
    selection_2 = scoretools.make_notes([4, 5], [Duration(1, 4)])
    selection_3 = scoretools.make_notes([7, 9], [Duration(1, 4)])
    selection_4 = scoretools.make_notes([11, 12], [Duration(1, 4)])
    selections = [selection_1, selection_2, selection_3, selection_4]
    container = Container()
    container.extend(selections)

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'4
            d'4
            e'4
            f'4
            g'4
            a'4
            b'4
            c''4
        }
        '''
        )

    assert inspect_(container).is_well_formed()



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_quantizationtools_BeatwiseQSchemaItem___new___01():

    item = quantizationtools.BeatwiseQSchemaItem()

    assert item.beatspan is None
    assert item.search_tree is None
    assert item.tempo is None


def test_quantizationtools_BeatwiseQSchemaItem___new___02():

    item = quantizationtools.BeatwiseQSchemaItem(
        tempo=((1, 4), 60)
        )

    assert item.beatspan is None
    assert item.search_tree is None
    assert item.tempo == Tempo((1, 4), 60)


def test_quantizationtools_BeatwiseQSchemaItem___new___03():

    item = quantizationtools.BeatwiseQSchemaItem(
        beatspan=(1, 8)
        )

    assert item.beatspan == durationtools.Duration(1, 8)
    assert item.search_tree is None
    assert item.tempo is None


def test_quantizationtools_BeatwiseQSchemaItem___new___04():

    item = quantizationtools.BeatwiseQSchemaItem(
        beatspan=(1, 8),
        tempo=((1, 4), 57),
        )

    assert item.beatspan == durationtools.Duration(1, 8)
    assert item.search_tree is None
    assert item.tempo == Tempo((1, 4), 57)


def test_quantizationtools_BeatwiseQSchemaItem___new___05():

    tempo = Tempo(textual_indication='lento')
    pytest.raises(
        AssertionError, 
        'item = quantizationtools.BeatwiseQSchemaItem(tempo=tempo)',
        )



# -*- coding: utf-8 -*-
import sys
import unicodedata


def strip_diacritics(binary_string):
    r'''Strips diacritics from `binary_string`.

    ..  container:: example

        ::

            >>> binary_string = 'Dvořák'

        ::

            >>> print(binary_string)
            Dvořák

        ::

            >>> stringtools.strip_diacritics(binary_string)
            'Dvorak'

    Returns ASCII string.
    '''
    if sys.version_info[0] < 3:
        unicode_string = unicode(binary_string, 'utf-8')
    else:
        unicode_string = binary_string
    normalized_unicode_string = unicodedata.normalize('NFKD', unicode_string)
    ascii_string = normalized_unicode_string.encode('ascii', 'ignore')
    if sys.version_info[0] < 3:
        return ascii_string
    else:
        return ascii_string.decode('utf-8')



# -*- coding: utf-8 -*-
from sphinx.util.compat import Directive
from sphinx.util.nodes import set_source_info


class RevealDirective(Directive):
    r'''An abjad-book reveal directive.

    Generates an ``abjad_reveal_block`` node.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Sphinx Internals'

    has_content = False
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {}

    ### PUBLIC METHODS ###

    def run(self):
        r'''Executes the directive.
        '''
        from abjad.tools import abjadbooktools
        block = abjadbooktools.abjad_reveal_block()
        block['reveal-label'] = self.arguments[0]
        set_source_info(self, block)
        return [block]



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedInterval___abs___01():

    interval = pitchtools.NamedInterval('minor', 3)
    assert abs(interval) == pitchtools.NamedInterval('minor', 3)


def test_pitchtools_NamedInterval___abs___02():

    interval = pitchtools.NamedInterval('minor', -3)
    assert abs(interval) == pitchtools.NamedInterval('minor', 3)



# -*- coding: utf-8 -*-
import pytest
from abjad.tools import stringtools


def test_to_tridirectional_ordinal_constant_01():

    assert stringtools.to_tridirectional_ordinal_constant('^') == Up
    assert stringtools.to_tridirectional_ordinal_constant('-') == Center
    assert stringtools.to_tridirectional_ordinal_constant('_') == Down
    assert stringtools.to_tridirectional_ordinal_constant(Up) == Up
    assert stringtools.to_tridirectional_ordinal_constant('default') == Center
    assert stringtools.to_tridirectional_ordinal_constant('neutral') == Center
    assert stringtools.to_tridirectional_ordinal_constant(Down) == Down
    assert stringtools.to_tridirectional_ordinal_constant(1) == Up
    assert stringtools.to_tridirectional_ordinal_constant(0) == Center
    assert stringtools.to_tridirectional_ordinal_constant(-1) == Down
    assert stringtools.to_tridirectional_ordinal_constant(None) is None
    assert pytest.raises(ValueError,
        "stringtools.to_tridirectional_ordinal_constant('foo')")



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Chord___setattr___01():
    r'''Slots constrain chord attributes.
    '''

    chord = Chord("<ef' cs' f''>4")

    assert pytest.raises(AttributeError, "chord.foo = 'bar'")



# -*- coding: utf-8 -*-


def tweak(argument):
    r'''Makes LilyPond tweak manager.

    ..  container:: example

        Tweaks markup:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> markup = Markup('Allegro assai', direction=Up)
            >>> tweak(markup).color = 'red'
            >>> attach(markup, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                    - \tweak color #red
                    ^ \markup { "Allegro assai" }
                d'4
                e'4
                f'4
            }

        Survives copy:

        ::

            >>> import copy
            >>> staff = Staff("c'4 d' e' f'")
            >>> markup_1 = Markup('Allegro assai', direction=Up)
            >>> tweak(markup_1).color = 'red'
            >>> markup_2 = copy.copy(markup_1)
            >>> attach(markup_2, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                    - \tweak color #red
                    ^ \markup { "Allegro assai" }
                d'4
                e'4
                f'4
            }

        Survives dot-chaining:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> markup = Markup('Allegro assai', direction=Up)
            >>> tweak(markup).color = 'red'
            >>> markup = markup.italic()
            >>> attach(markup, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                    - \tweak color #red
                    ^ \markup {
                        \italic
                            "Allegro assai"
                        }
                d'4
                e'4
                f'4
            }

        Works for opposite-directed coincident markup:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> markup_1 = Markup('Allegro assai ...', direction=Up)
            >>> tweak(markup_1).color = 'red'
            >>> attach(markup_1, staff[0])
            >>> markup_2 = Markup('... ma non troppo', direction=Down)
            >>> tweak(markup_2).color = 'blue'
            >>> tweak(markup_2).staff_padding = 4
            >>> attach(markup_2, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                    - \tweak color #red
                    ^ \markup { "Allegro assai ..." }
                    - \tweak color #blue
                    - \tweak staff-padding #4
                    _ \markup { "... ma non troppo" }
                d'4
                e'4
                f'4
            }

        Ignored for same-directed coincident markup:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> markup_1 = Markup('Allegro assai ...', direction=Up)
            >>> tweak(markup_1).color = 'red'
            >>> attach(markup_1, staff[0])
            >>> markup_2 = Markup('... ma non troppo', direction=Up)
            >>> tweak(markup_2).color = 'blue'
            >>> tweak(markup_2).staff_padding = 4
            >>> attach(markup_2, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                    ^ \markup {
                        \column
                            {
                                \line
                                    {
                                        "Allegro assai ..."
                                    }
                                \line
                                    {
                                        "... ma non troppo"
                                    }
                            }
                        }
                d'4
                e'4
                f'4
            }

        ..  todo:: Remove courtesy autocolumn in favor of explicit tweaks?

    ..  container:: example

        Returns LilyPond tweak manager:

        ::

            >>> tweak(markup_1)
            LilyPondTweakManager(('color', 'red'))

    ''' 
    from abjad.tools import lilypondnametools
    if getattr(argument, '_lilypond_tweak_manager', None) is None:
        manager = lilypondnametools.LilyPondTweakManager()
        argument._lilypond_tweak_manager = manager
    return argument._lilypond_tweak_manager



# -*- coding: utf-8 -*-


def detach(prototype, component_expression=None):
    r'''Detaches `prototype` indicators from `component_expression`.

    ..  container:: example

        Detaches articulations from first note in staff:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> attach(Articulation('>'), staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4 -\accent
                d'4
                e'4
                f'4
            }

        ::

            >>> detach(Articulation, staff[0])
            (Articulation('>'),)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }

    Returns tuple of zero or more detached items.
    '''
    from abjad.tools import scoretools
    from abjad.tools import spannertools
    from abjad.tools import topleveltools
    if isinstance(prototype, spannertools.Spanner):
        prototype._detach()
        return
    assert component_expression is not None
    spanners = []
    grace_containers = []
    inspector = topleveltools.inspect_(component_expression)
    if isinstance(prototype, type):
        if issubclass(prototype, spannertools.Spanner):
            spanners = inspector.get_spanners(prototype)
        elif issubclass(prototype, scoretools.GraceContainer):
            grace_containers = inspector.get_grace_containers(prototype)
        else:
            assert hasattr(component_expression, '_indicator_expressions')
            result = []
            for x in component_expression._indicator_expressions[:]:
                if isinstance(x, prototype):
                    component_expression._indicator_expressions.remove(x)
                    result.append(x)
                # indicator is a expression
                elif (hasattr(x, 'indicator') and
                    isinstance(x.indicator, prototype)):
                    x._detach()
                    result.append(x.indicator)
            result = tuple(result)
            return result
    else:
        if isinstance(prototype, spannertools.Spanner):
            spanners = inspector.get_spanners(prototype)
        elif isinstance(prototype, scoretools.GraceContainer):
            grace_containers = inspector.get_grace_containers(
                kind=prototype.kind,
                )
        else:
            assert hasattr(component_expression, '_indicator_expressions')
            result = []
            for x in component_expression._indicator_expressions[:]:
                if x == prototype:
                    component_expression._indicator_expressions.remove(x)
                    result.append(x)
                # indicator is an expression
                elif hasattr(x, 'indicator') and x.indicator == prototype:
                    x._detach()
                    result.append(x.indicator)
            result = tuple(result)
            return result
    items = []
    items.extend(spanners)
    items.extend(grace_containers)
    for item in items:
        item._detach()
    items = tuple(items)
    return items



# -*- coding: utf-8 -*-
import pytest
from abjad import *


@pytest.mark.skip('Pending 2.19.24 update.')
def test_lilypondparsertools_LilyPondParser__lilypondfile__HeaderBlock_01():
    string = r'''
    globalvariable = "This is a global variable."
    \header {
        globalvariable = "This overrides the global variable"
        localvariable = "and this is a local variable."
        something = #4
        title = \markup { \globalvariable \localvariable }
    }
    \score {
        \new Staff { c'4 ^ \markup { \globalvariable } }
    }
    '''
    result = parse(string)
    assert isinstance(result, lilypondfiletools.LilyPondFile)
    assert len(result.items) == 2
    assert format(result.items[0]) == stringtools.normalize(
        r'''
        \header {
            globalvariable = #"This overrides the global variable"
            localvariable = #"and this is a local variable."
            something = #4
            title = \markup {
                "This overrides the global variable"
                "and this is a local variable."
                }
        }
        '''
        )
    assert format(result.items[1]) == stringtools.normalize(
        r'''
        \score {
            \new Staff {
                c'4 ^ \markup { This is a global variable. }
            }
        }
        '''
        )


@pytest.mark.skip('Pending 2.19.24 update.')
def test_lilypondparsertools_LilyPondParser__lilypondfile__HeaderBlock_02():
    string = r'''
    \header {
        composername = "Foo von Bar"
        composer = \markup { by \bold \composername }
        title = \markup { The ballad of \composername }
        tagline = \markup { "" }
    }
    {
        c'1
    }
    '''
    result = parse(string)
    assert isinstance(result, lilypondfiletools.LilyPondFile)
    assert len(result.items) == 2
    assert format(result.items[0]) == stringtools.normalize(
        r'''
        \header {
            composer = \markup {
                by
                \bold
                    "Foo von Bar"
                }
            composername = #"Foo von Bar"
            tagline = ##f
            title = \markup {
                The
                ballad
                of
                "Foo von Bar"
                }
        }
        ''')
    assert format(result.items[1]) == stringtools.normalize(
        r'''
        {
            c'1
        }
        ''')



# -*- coding: utf-8 -*-
from abjad import *


def test_metertools_Meter___iter___01():

    bh = metertools.Meter(TimeSignature((3, 8)))

    result = [x for x in bh]

    assert result == [
        (durationtools.Offset(0, 1), durationtools.Offset(1, 8)),
        (durationtools.Offset(1, 8), durationtools.Offset(1, 4)),
        (durationtools.Offset(1, 4), durationtools.Offset(3, 8)),
        (durationtools.Offset(0, 1), durationtools.Offset(3, 8))
    ]



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_transpose_pitch_carrier_by_interval_01():

    pitch = NamedPitch(12)

    named_interval = pitchtools.NamedInterval('minor', 2)
    transposed = pitchtools.transpose_pitch_carrier_by_interval(pitch, named_interval)

    assert transposed == NamedPitch('df', 5)


def test_pitchtools_transpose_pitch_carrier_by_interval_02():

    pitch = NamedPitch(12)

    numbered_interval = pitchtools.NumberedInterval(1)
    transposed = pitchtools.transpose_pitch_carrier_by_interval(pitch, numbered_interval)

    assert transposed == NamedPitch('cs', 5)


def test_pitchtools_transpose_pitch_carrier_by_interval_03():
    r'''Transpose pitch.
    '''

    pitch = NamedPitch(12)
    interval = pitchtools.NumberedInterval(-3)
    new = pitchtools.transpose_pitch_carrier_by_interval(pitch, interval)
    assert new == NamedPitch(9)
    assert new is not pitch


def test_pitchtools_transpose_pitch_carrier_by_interval_04():
    r'''Transpose note.
    '''

    note = Note(12, (1, 4))
    interval = pitchtools.NumberedInterval(-3)
    new = pitchtools.transpose_pitch_carrier_by_interval(note, interval)
    assert new.written_pitch == NamedPitch(9)
    assert new is not note


def test_pitchtools_transpose_pitch_carrier_by_interval_05():
    r'''Transpose chord.
    '''

    chord = Chord([12, 13, 14], (1, 4))
    interval = pitchtools.NumberedInterval(-3)
    new = pitchtools.transpose_pitch_carrier_by_interval(chord, interval)
    assert new.written_pitches == tuple([NamedPitch(x) for x in [9, 10, 11]])
    assert new is not chord


def test_pitchtools_transpose_pitch_carrier_by_interval_06():

    pitch = NamedPitch(12)
    mdi = pitchtools.NamedInterval('minor', -3)

    transposed_pitch = pitchtools.transpose_pitch_carrier_by_interval(pitch, mdi)
    assert transposed_pitch == NamedPitch('a', 4)


def test_pitchtools_transpose_pitch_carrier_by_interval_07():
    r'''Retun non-pitch-carrying input changed.
    '''

    rest = Rest('r4')

    assert pitchtools.transpose_pitch_carrier_by_interval(rest, '+m2') is rest



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_TonalAnalysisAgent_analyze_tonal_functions_01():

    key_signature = KeySignature('c', 'major')
    chord = Chord('<c e g>4')
    tonal_function = tonalanalysistools.RomanNumeral(1, 'major', 5, 0)
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_tonal_functions(key_signature)
    assert result == [tonal_function]

    chord = Chord(['e', 'g', "c'"], (1, 4))
    tonal_function = tonalanalysistools.RomanNumeral(1, 'major', 5, 1)
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_tonal_functions(key_signature)
    assert result == [tonal_function]

    chord = Chord(['g', "c'", "e'"], (1, 4))
    tonal_function = tonalanalysistools.RomanNumeral(1, 'major', 5, 2)
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_tonal_functions(key_signature)
    assert result == [tonal_function]


def test_tonalanalysistools_TonalAnalysisAgent_analyze_tonal_functions_02():

    key_signature = KeySignature('c', 'major')
    chord = Chord(['c', 'ef', 'g'], (1, 4))
    tonal_function = tonalanalysistools.RomanNumeral(1, 'minor', 5, 0)
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_tonal_functions(key_signature)
    assert result == [tonal_function]

    chord = Chord(['ef', 'g', "c'"], (1, 4))
    tonal_function = tonalanalysistools.RomanNumeral(1, 'minor', 5, 1)
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_tonal_functions(key_signature)
    assert result == [tonal_function]

    chord = Chord(['g', "c'", "ef'"], (1, 4))
    tonal_function = tonalanalysistools.RomanNumeral(1, 'minor', 5, 2)
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_tonal_functions(key_signature)
    assert result == [tonal_function]


def test_tonalanalysistools_TonalAnalysisAgent_analyze_tonal_functions_03():

    key_signature = KeySignature('c', 'major')
    chord = Chord(['c', 'e', 'g', 'bf'], (1, 4))
    tonal_function = tonalanalysistools.RomanNumeral(1, 'dominant', 7, 0)
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_tonal_functions(key_signature)
    assert result == [tonal_function]

    chord = Chord(['e', 'g', 'bf', "c'"], (1, 4))
    tonal_function = tonalanalysistools.RomanNumeral(1, 'dominant', 7, 1)
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_tonal_functions(key_signature)
    assert result == [tonal_function]

    chord = Chord(['g', 'bf', "c'", "e'"], (1, 4))
    tonal_function = tonalanalysistools.RomanNumeral(1, 'dominant', 7, 2)
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_tonal_functions(key_signature)
    assert result == [tonal_function]

    chord = Chord(['bf', "c'", "e'", "g'"], (1, 4))
    tonal_function = tonalanalysistools.RomanNumeral(1, 'dominant', 7, 3)
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_tonal_functions(key_signature)
    assert result == [tonal_function]


def test_tonalanalysistools_TonalAnalysisAgent_analyze_tonal_functions_04():

    key_signature = KeySignature('c', 'major')
    chord = Chord(['c', 'cs', 'd'], (1, 4))
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_tonal_functions(key_signature)
    assert result == [None]



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Selection_group_by_01():

    leaves = scoretools.make_leaves([0, 2, 4, None, None, 5, 7], [(1, 8)])
    staff = Staff(leaves)
    groups = leaves.group_by(type)

    assert groups == [
        (staff[0], staff[1], staff[2]),
        (staff[3], staff[4]),
        (staff[5], staff[6])
        ]


# -*- coding: utf-8 -*-
from docutils.parsers.rst import directives
from sphinx.util.compat import Directive
from sphinx.util.nodes import set_source_info


class ImportDirective(Directive):
    r'''An abjad-book import directive.

    Represents a class or function to be imported into an interactive session.

    Generates an ``abjad_import_block`` node.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Sphinx Internals'

    has_content = False
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        'hide': directives.flag,
        'reveal-label': str,
        }

    ### PUBLIC METHODS ###

    def run(self):
        r'''Executes the directive.
        '''
        from abjad.tools import abjadbooktools
        path = self.arguments[0]
        block = abjadbooktools.abjad_import_block()
        block['path'] = path
        if 'hide' in self.options:
            block['hide'] = True
        if 'reveal-label' in self.options:
            block['reveal-label'] = self.options.get('reveal-label')
        set_source_info(self, block)
        return [block]



# -*- coding: utf-8 -*-
from abjad import *
import copy


def test_pitchtools_NamedPitchClass___copy___01():

    npc = pitchtools.NamedPitchClass('cs')
    new = copy.copy(npc)

    assert new == npc
    assert new is not npc



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TreeContainer_children_01():

    leaf_a = datastructuretools.TreeNode()
    leaf_b = datastructuretools.TreeNode()
    leaf_c = datastructuretools.TreeNode()
    subcontainer = datastructuretools.TreeContainer([leaf_b, leaf_c])
    leaf_d = datastructuretools.TreeNode()

    container = datastructuretools.TreeContainer([leaf_a, subcontainer, leaf_d])

    assert container.children == (leaf_a, subcontainer, leaf_d)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedInterval___lt___01():
    r'''Compare two ascending numbered intervals.
    '''

    interval_1 = pitchtools.NumberedInterval(2)
    interval_2 = pitchtools.NumberedInterval(6)

    assert interval_1 < interval_2
    assert not interval_2 < interval_1


def test_pitchtools_NumberedInterval___lt___02():
    r'''Compare two descending numbered intervals.
    '''

    interval_1 = pitchtools.NumberedInterval(-2)
    interval_2 = pitchtools.NumberedInterval(-6)

    assert interval_1 < interval_2
    assert not interval_2 < interval_1



# -*- coding: utf-8 -*-


def make_spacing_vector(
    basic_distance,
    minimum_distance,
    padding,
    stretchability,
    ):
    r'''Makes spacing vector.

    ::

        >>> vector = schemetools.make_spacing_vector(0, 0, 12, 0)

    Use to set paper block spacing attributes:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> lilypond_file = LilyPondFile.new(staff)
        >>> spacing_vector = schemetools.make_spacing_vector(0, 0, 12, 0)
        >>> lilypond_file.paper_block.system_system_spacing = spacing_vector

    ..  doctest::

        >>> print(format(lilypond_file)) # doctest: +SKIP
        % Abjad revision 4229
        % 2011-04-07 15:19

        \version "2.13.44"
        \language "english"

        \paper {
            system-system-spacing = #'(
                (basic-distance . 0) (minimum-distance . 0) (padding . 12) (stretchability . 0))
        }

        \score {
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
        }

    Returns scheme vector.
    '''
    from abjad.tools import schemetools
    return schemetools.SchemeVector(
        schemetools.SchemePair('basic-distance', basic_distance),
        schemetools.SchemePair('minimum-distance', minimum_distance),
        schemetools.SchemePair('padding', padding),
        schemetools.SchemePair('stretchability', stretchability))



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSet_multiply_01():

    assert pitchtools.PitchClassSet([0, 1, 5]).multiply(5) == \
        pitchtools.PitchClassSet([0, 1, 5])
    assert pitchtools.PitchClassSet([1, 2, 6]).multiply(5) == \
        pitchtools.PitchClassSet([5, 6, 10])
    assert pitchtools.PitchClassSet([2, 3, 7]).multiply(5) == \
        pitchtools.PitchClassSet([3, 10, 11])



# -*- coding: utf-8 -*-
import os
import platform
from abjad.tools import commandlinetools
from abjad.tools import stringtools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase
try:
    from unittest import mock
except ImportError:
    import mock


class Test(ScorePackageScriptTestCase):

    expected_files = [
        'test_score/test_score/segments/.gitignore',
        'test_score/test_score/segments/__init__.py',
        'test_score/test_score/segments/metadata.json',
        'test_score/test_score/segments/test_segment/__init__.py',
        'test_score/test_score/segments/test_segment/definition.py',
        'test_score/test_score/segments/test_segment/illustration.ly',
        'test_score/test_score/segments/test_segment/illustration.pdf',
        'test_score/test_score/segments/test_segment/metadata.json',
        ]

    if platform.system().lower() == 'windows':
        expected_files = [_.replace('/', os.path.sep) for _ in expected_files]

    expected_illustration_contents = stringtools.normalize(
        r'''
        \language "english"

        \include "../../stylesheets/stylesheet.ily"

        \header {}

        \layout {}

        \paper {}

        \score {
            \context Score = "Example Score" <<
                \context Staff = "Example Staff" {
                    \context Voice = "Example Voice" {
                        c'4 (
                        d'4
                        e'4
                        f'4 )
                    }
                }
            >>
        }
        ''')

    def test_lilypond_error(self):
        """
        Handle failing LilyPond rendering.
        """
        self.create_score()
        segment_path = self.create_segment('test_segment')
        definition_path = segment_path.joinpath('definition.py')
        with open(str(definition_path), 'w') as file_pointer:
            file_pointer.write(stringtools.normalize(r'''
            # -*- coding: utf-8 -*-
            from abjad.tools import abctools
            from abjad.tools import lilypondfiletools
            from abjad.tools import scoretools


            class FaultySegmentMaker(abctools.AbjadObject):

                def __call__(
                    self,
                    segment_metadata=None,
                    previous_segment_metadata=None,
                    ):
                    lilypond_file = lilypondfiletools.LilyPondFile.new(
                        scoretools.Staff("c'4 ( d'4 e'4 f'4 )")
                        )
                    lilypond_file.items.append(r'\this-does-not-exist')
                    return lilypond_file, segment_metadata

            segment_maker = FaultySegmentMaker()
            '''))
        script = commandlinetools.ManageSegmentScript()
        command = ['--illustrate', 'test_segment']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
        self.compare_captured_output(r'''
            Illustration candidates: 'test_segment' ...
                Reading test_score/segments/metadata.json ... OK!
            Illustrating test_score/segments/test_segment/
                Reading test_score/segments/metadata.json ... OK!
                Reading test_score/segments/test_segment/metadata.json ... JSON does not exist.
                Importing test_score.segments.test_segment.definition
                Writing test_score/segments/test_segment/metadata.json
                    Abjad runtime: ... second...
                Writing test_score/segments/test_segment/illustration.ly ... OK!
                Writing test_score/segments/test_segment/illustration.pdf ... Failed!
        '''.replace('/', os.path.sep))
        illustration_ly_path = segment_path.joinpath('illustration.ly')
        assert illustration_ly_path.exists()
        self.compare_lilypond_contents(
            illustration_ly_path, stringtools.normalize(r'''
            \language "english"

            \header {}

            \layout {}

            \paper {}

            \score {
                \new Staff {
                    c'4 (
                    d'4
                    e'4
                    f'4 )
                }
            }

            \this-does-not-exist
            '''))

    def test_missing_definition(self):
        """
        Handle missing definition.
        """
        self.create_score()
        segment_path = self.create_segment('test_segment')
        definition_path = segment_path.joinpath('definition.py')
        definition_path.unlink()
        script = commandlinetools.ManageSegmentScript()
        command = ['--illustrate', 'test_segment']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
        self.compare_captured_output(r'''
            Illustration candidates: 'test_segment' ...
                Reading test_score/segments/metadata.json ... OK!
            Illustrating test_score/segments/test_segment/
                Reading test_score/segments/metadata.json ... OK!
                Reading test_score/segments/test_segment/metadata.json ... JSON does not exist.
                Importing test_score.segments.test_segment.definition
        '''.replace('/', os.path.sep))

    def test_python_error_on_illustrate(self):
        """
        Handle exceptions inside the Python module on __call__().
        """
        self.create_score()
        segment_path = self.create_segment('test_segment')
        definition_path = segment_path.joinpath('definition.py')
        with open(str(definition_path), 'w') as file_pointer:
            file_pointer.write(stringtools.normalize(r'''
            # -*- coding: utf-8 -*-
            from abjad.tools import abctools


            class FaultySegmentMaker(abctools.AbjadObject):

                def __call__(
                    self,
                    segment_metadata=None,
                    previous_segment_metadata=None,
                    ):
                    raise TypeError('This is intentionally broken.')

            segment_maker = FaultySegmentMaker()
            '''))
        script = commandlinetools.ManageSegmentScript()
        command = ['--illustrate', 'test_segment']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
        self.compare_captured_output(r'''
            Illustration candidates: 'test_segment' ...
                Reading test_score/segments/metadata.json ... OK!
            Illustrating test_score/segments/test_segment/
                Reading test_score/segments/metadata.json ... OK!
                Reading test_score/segments/test_segment/metadata.json ... JSON does not exist.
                Importing test_score.segments.test_segment.definition
        '''.replace('/', os.path.sep))

    def test_python_error_on_import(self):
        """
        Handle exceptions inside the Python module on import.
        """
        self.create_score()
        segment_path = self.create_segment('test_segment')
        definition_path = segment_path.joinpath('definition.py')
        with open(str(definition_path), 'a') as file_pointer:
            file_pointer.write('\n\nfailure = 1 / 0\n')
        script = commandlinetools.ManageSegmentScript()
        command = ['--illustrate', 'test_segment']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                with self.assertRaises(SystemExit) as context_manager:
                    script(command)
                assert context_manager.exception.code == 1
        self.compare_captured_output(r'''
            Illustration candidates: 'test_segment' ...
                Reading test_score/segments/metadata.json ... OK!
            Illustrating test_score/segments/test_segment/
                Reading test_score/segments/metadata.json ... OK!
                Reading test_score/segments/test_segment/metadata.json ... JSON does not exist.
                Importing test_score.segments.test_segment.definition
        '''.replace('/', os.path.sep))

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_all_segments(self, open_file_mock):
        self.create_score()
        self.create_segment('segment_one')
        self.create_segment('segment_two')
        self.create_segment('segment_three')
        script = commandlinetools.ManageSegmentScript()
        command = ['--illustrate', '*']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
            Illustration candidates: '*' ...
                Reading test_score/segments/metadata.json ... OK!
            Illustrating test_score/segments/segment_one/
                Reading test_score/segments/metadata.json ... OK!
                Reading test_score/segments/segment_one/metadata.json ... JSON does not exist.
                Importing test_score.segments.segment_one.definition
                Writing test_score/segments/segment_one/metadata.json
                    Abjad runtime: ... second...
                Writing test_score/segments/segment_one/illustration.ly ... OK!
                Writing test_score/segments/segment_one/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/segments/segment_one/
            Illustrating test_score/segments/segment_two/
                Reading test_score/segments/metadata.json ... OK!
                Reading test_score/segments/segment_one/metadata.json ... OK!
                Reading test_score/segments/segment_two/metadata.json ... JSON does not exist.
                Importing test_score.segments.segment_two.definition
                Writing test_score/segments/segment_two/metadata.json
                    Abjad runtime: ... second...
                Writing test_score/segments/segment_two/illustration.ly ... OK!
                Writing test_score/segments/segment_two/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/segments/segment_two/
            Illustrating test_score/segments/segment_three/
                Reading test_score/segments/metadata.json ... OK!
                Reading test_score/segments/segment_two/metadata.json ... OK!
                Reading test_score/segments/segment_three/metadata.json ... JSON does not exist.
                Importing test_score.segments.segment_three.definition
                Writing test_score/segments/segment_three/metadata.json
                    Abjad runtime: ... second...
                Writing test_score/segments/segment_three/illustration.ly ... OK!
                Writing test_score/segments/segment_three/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/segments/segment_three/
        '''.replace('/', os.path.sep))
        assert self.segments_path.joinpath(
            'segment_one',
            'illustration.pdf',
            ).exists()
        assert self.segments_path.joinpath(
            'segment_two',
            'illustration.pdf',
            ).exists()
        assert self.segments_path.joinpath(
            'segment_three',
            'illustration.pdf',
            ).exists()

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_filtered_segments(self, open_file_mock):
        self.create_score()
        self.create_segment('segment_one')
        self.create_segment('segment_two')
        self.create_segment('segment_three')
        script = commandlinetools.ManageSegmentScript()
        command = ['--illustrate', 'segment_t*']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
            Illustration candidates: 'segment_t*' ...
                Reading test_score/segments/metadata.json ... OK!
            Illustrating test_score/segments/segment_two/
                Reading test_score/segments/metadata.json ... OK!
                Reading test_score/segments/segment_one/metadata.json ... JSON does not exist.
                Reading test_score/segments/segment_two/metadata.json ... JSON does not exist.
                Importing test_score.segments.segment_two.definition
                Writing test_score/segments/segment_two/metadata.json
                    Abjad runtime: ... second...
                Writing test_score/segments/segment_two/illustration.ly ... OK!
                Writing test_score/segments/segment_two/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/segments/segment_two/
            Illustrating test_score/segments/segment_three/
                Reading test_score/segments/metadata.json ... OK!
                Reading test_score/segments/segment_two/metadata.json ... OK!
                Reading test_score/segments/segment_three/metadata.json ... JSON does not exist.
                Importing test_score.segments.segment_three.definition
                Writing test_score/segments/segment_three/metadata.json
                    Abjad runtime: ... second...
                Writing test_score/segments/segment_three/illustration.ly ... OK!
                Writing test_score/segments/segment_three/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/segments/segment_three/
        '''.replace('/', os.path.sep))
        assert not self.segments_path.joinpath(
            'segment_one',
            'illustration.pdf',
            ).exists()
        assert self.segments_path.joinpath(
            'segment_two',
            'illustration.pdf',
            ).exists()
        assert self.segments_path.joinpath(
            'segment_three',
            'illustration.pdf',
            ).exists()

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_one_segment(self, open_file_mock):
        self.create_score()
        self.create_segment('test_segment')
        script = commandlinetools.ManageSegmentScript()
        command = ['--illustrate', 'test_segment']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
            Illustration candidates: 'test_segment' ...
                Reading test_score/segments/metadata.json ... OK!
            Illustrating test_score/segments/test_segment/
                Reading test_score/segments/metadata.json ... OK!
                Reading test_score/segments/test_segment/metadata.json ... JSON does not exist.
                Importing test_score.segments.test_segment.definition
                Writing test_score/segments/test_segment/metadata.json
                    Abjad runtime: ... second...
                Writing test_score/segments/test_segment/illustration.ly ... OK!
                Writing test_score/segments/test_segment/illustration.pdf ... OK!
                    LilyPond runtime: ... second...
                Illustrated test_score/segments/test_segment/
        '''.replace('/', os.path.sep))
        self.compare_path_contents(self.segments_path, self.expected_files)
        illustration_path = self.segments_path.joinpath(
            'test_segment', 'illustration.ly')
        self.compare_lilypond_contents(
            illustration_path,
            self.expected_illustration_contents,
            )



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Selection__attach_tie_spanner_to_leaf_pair_01():
    r'''Span left leaf with spanner and right leaf without spanner.
    '''

    voice = Voice("c'8 c'8 c'8 c'8")
    tie = spannertools.Tie()
    attach(tie, voice[:2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 ~
            c'8
            c'8
            c'8
        }
        '''
        )

    selector = select().by_leaf(flatten=True)
    leaves = selector(voice)
    leaves[1:3]._attach_tie_spanner_to_leaf_pair()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 ~
            c'8 ~
            c'8
            c'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_selectiontools_Selection__attach_tie_spanner_to_leaf_pair_02():
    r'''Span left leaf with spanner and right leaf with spanner.
    '''

    voice = Voice("c'8 c'8 c'8 c'8")
    tie = spannertools.Tie()
    attach(tie, voice[:2])
    tie = spannertools.Tie()
    attach(tie, voice[2:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 ~
            c'8
            c'8 ~
            c'8
        }
        '''
        )

    selector = select().by_leaf(flatten=True)
    leaves = selector(voice)
    leaves[1:3]._attach_tie_spanner_to_leaf_pair()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 ~
            c'8 ~
            c'8 ~
            c'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_selectiontools_Selection__attach_tie_spanner_to_leaf_pair_03():
    r'''Span left leaves with no spanner.
    '''

    voice = Voice("c'8 c'8 c'8 c'8")
    selector = select().by_leaf(flatten=True)
    leaves = selector(voice)
    leaves[1:3]._attach_tie_spanner_to_leaf_pair()

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8
            c'8 ~
            c'8
            c'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


# -*- coding: utf-8 -*-
from abjad.tools import rhythmtreetools
import pickle


def test_rhythmtreetools_RhythmTreeNode_pickle_01():

    string = '(1 (1 (2 (1 1 1)) 2))'
    tree = rhythmtreetools.RhythmTreeParser()(string)[0]

    pickled = pickle.loads(pickle.dumps(tree))

    assert format(pickled) == format(tree)
    assert pickled != tree
    assert pickled is not tree



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_HiddenStaffSpanner___init___01():
    r'''Initialize empty hidden staff spanner.
    '''

    spanner = spannertools.HiddenStaffSpanner()
    assert isinstance(spanner, spannertools.HiddenStaffSpanner)


def test_spannertools_HiddenStaffSpanner___init___02():
    r'''Hide staff around one measure.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")
    leaves = select(staff).by_leaf()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }
        '''
        )

    spanner = spannertools.HiddenStaffSpanner()
    attach(spanner, leaves[2:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \stopStaff
                e'8
                f'8
                \startStaff
            }
            {
                g'8
                a'8
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_HiddenStaffSpanner___init___03():
    r'''Hide staff around one leaf.
    '''

    note = Note(0, (1, 8))
    spanner = spannertools.HiddenStaffSpanner()
    attach(spanner, note)

    assert format(note) == stringtools.normalize(
        r'''
        \stopStaff
        c'8
        \startStaff
        '''
        )

    assert inspect_(note).is_well_formed()



# -*- coding: utf-8 -*-
import numbers
from abjad import Fraction


def divide_number_by_ratio(number, ratio):
    r'''Divides `number` by `ratio`.

    ..  container:: example

        ::

            >>> mathtools.divide_number_by_ratio(1, [1, 1, 3])
            [Fraction(1, 5), Fraction(1, 5), Fraction(3, 5)]

        ::

            >>> mathtools.divide_number_by_ratio(Fraction(1), [1, 1, 3])
            [Fraction(1, 5), Fraction(1, 5), Fraction(3, 5)]

        ::

            >>> mathtools.divide_number_by_ratio(1.0, [1, 1, 3]) # doctest: +SKIP
            [0.20000000000000001, 0.20000000000000001, 0.60000000000000009]

    Returns list of fractions or list of floats.
    '''
    from abjad.tools import mathtools
    assert isinstance(number, numbers.Number)
    ratio = mathtools.Ratio(ratio)
    denominator = sum(ratio.numbers)
    factors = [Fraction(_, denominator) for _ in ratio.numbers]
    result = [_ * number for _ in factors]
    return result



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class ChordQuality(AbjadObject):
    '''A chord quality, such as major, minor, dominant,
    diminished and so on.

    Value object that can not be changed after instantiation.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_quality_string',
        )

    _acceptable_quality_strings = (
        'augmented',
        'diminished',
        'dominant',
        'half diminished',
        'major',
        'minor',
        )

    _uppercase_quality_strings = (
        'augmented',
        'dominant',
        'major',
        )

    ### INITIALIZER ###

    def __init__(self, quality_string='major'):
        if quality_string not in self._acceptable_quality_strings:
            message = 'can not initialize chord quality: {!r}.'
            message = message.format(quality_string)
            raise ValueError(message)
        self._quality_string = quality_string

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a chord quality with quality string equal to
        that of this chord quality. Otherwise false.

        Returns true or false.
        '''
        if isinstance(arg, type(self)):
            if self.quality_string == arg.quality_string:
                return True
        return False

    def __hash__(self):
        r'''Hashes chord quality.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(ChordQuality, self).__hash__()

    def __ne__(self, arg):
        r'''Is true when chord quality does not equal `arg`. Otherwise false.

        Returns true or false.
        '''
        return not self == arg

    def __repr__(self):
        r'''Gets interpreter representation of chord quality.

        Returns string.
        '''
        return '{}({})'.format(type(self).__name__, self.quality_string)

    ### PUBLIC PROPERTIES ###

    @property
    def is_uppercase(self):
        r'''Is true when chord quality is uppercase. Otherwise false.

        Returns true or false.
        '''
        return self.quality_string in self._uppercase_quality_strings

    @property
    def quality_string(self):
        r'''Quality string of chord quality.

        Returns string.
        '''
        return self._quality_string



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_fill_measures_with_full_measure_spacer_skips_01():
    r'''Populates non-power-of-two measure with time-scaled skip.
    '''

    measure = Measure((5, 18), [])
    measure.implicit_scaling = True
    scoretools.fill_measures_with_full_measure_spacer_skips(measure)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 5/18
            \scaleDurations #'(8 . 9) {
                s1 * 5/16
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_fill_measures_with_full_measure_spacer_skips_02():
    r'''Populates measures conditionally.

    Iteration control tests index of iteration.
    '''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    staff.extend("abj: | 2/8 g'8 a'8 || 2/8 b'8 c''8 |")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    def iterctrl(measure, i):
        return i % 2 == 1

    scoretools.fill_measures_with_full_measure_spacer_skips(
        staff, iterctrl)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                s1 * 1/4
            }
            {
                g'8
                a'8
            }
            {
                s1 * 1/4
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_fill_measures_with_full_measure_spacer_skips_03():
    r'''Populates measures conditionally.

    Iteration control tests measure length.
    '''

    staff = Staff([
        Measure((2, 8), "c'8 d'8"),
        Measure((3, 8), "c'8 d'8 e'8"),
        Measure((4, 8), "c'8 d'8 e'8 f'8"),
        ])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \time 3/8
                c'8
                d'8
                e'8
            }
            {
                \time 4/8
                c'8
                d'8
                e'8
                f'8
            }
        }
        '''
        )

    scoretools.fill_measures_with_full_measure_spacer_skips(
        staff,
        lambda m, i: 2 < len(m),
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \time 3/8
                s1 * 3/8
            }
            {
                \time 4/8
                s1 * 1/2
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.abctools import Parser


class RhythmTreeParser(Parser):
    r'''A rhythm-tree syntax parser.

    Abjad’s rhythm-tree parser parses a micro-language resembling Ircam’s RTM
    Lisp syntax, and generates a sequence of RhythmTree structures, which can
    be furthered manipulated by composers, before being converted into an Abjad
    score object:

    ::

        >>> parser = rhythmtreetools.RhythmTreeParser()

    ::

        >>> string = '(3 (1 (1 ((2 (1 1 1)) 2 2 1))))'
        >>> rhythm_tree_list = parser(string)
        >>> rhythm_tree_container = rhythm_tree_list[0]
        >>> rhythm_tree_container.rtm_format
        '(3 (1 (1 ((2 (1 1 1)) 2 2 1))))'

    ::

        >>> print(format(rhythm_tree_container))
        rhythmtreetools.RhythmTreeContainer(
            children=(
                rhythmtreetools.RhythmTreeLeaf(
                    preprolated_duration=durationtools.Duration(1, 1),
                    is_pitched=True,
                    ),
                rhythmtreetools.RhythmTreeContainer(
                    children=(
                        rhythmtreetools.RhythmTreeContainer(
                            children=(
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=durationtools.Duration(1, 1),
                                    is_pitched=True,
                                    ),
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=durationtools.Duration(1, 1),
                                    is_pitched=True,
                                    ),
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=durationtools.Duration(1, 1),
                                    is_pitched=True,
                                    ),
                                ),
                            preprolated_duration=durationtools.Duration(2, 1),
                            ),
                        rhythmtreetools.RhythmTreeLeaf(
                            preprolated_duration=durationtools.Duration(2, 1),
                            is_pitched=True,
                            ),
                        rhythmtreetools.RhythmTreeLeaf(
                            preprolated_duration=durationtools.Duration(2, 1),
                            is_pitched=True,
                            ),
                        rhythmtreetools.RhythmTreeLeaf(
                            preprolated_duration=durationtools.Duration(1, 1),
                            is_pitched=True,
                            ),
                        ),
                    preprolated_duration=durationtools.Duration(1, 1),
                    ),
                ),
            preprolated_duration=durationtools.Duration(3, 1),
            )

    ::

        >>> base_duration = (1, 4)
        >>> component_list = rhythm_tree_container(base_duration)
        >>> tuplet = component_list[0]
        >>> show(tuplet) # doctest: +SKIP

    ..  doctest::

        >>> print(format(tuplet))
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/4 {
            c'2
            \times 4/7 {
                \times 2/3 {
                    c'8
                    c'8
                    c'8
                }
                c'4
                c'4
                c'8
            }
        }

    Returns `RhythmTreeParser` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PUBLIC PROPERTIES ###

    @property
    def lexer_rules_object(self):
        return self

    @property
    def parser_rules_object(self):
        return self

    ### LEX SETUP ###

    tokens = (
        'DURATION',
        'LPAREN',
        'RPAREN'
    )

    t_LPAREN = r'\('
    t_RPAREN = r'\)'
    t_ignore = ' \n\t\r'

    ### YACC SETUP ###

    start = 'toplevel'

    ### LEX METHODS ###

    def t_DURATION(self, t):
        r'-?[1-9]\d*(/[1-9]\d*)?'
        parts = t.value.partition('/')
        if not parts[2]:
            t.value = durationtools.Duration(int(parts[0]))
        else:
            numerator, denominator = int(parts[0]), int(parts[2])
            fraction = mathtools.NonreducedFraction(numerator, denominator)
            preprolated_duration = durationtools.Duration(fraction)
            if fraction.numerator == preprolated_duration.numerator:
                t.value = preprolated_duration
            else:
                t.value = fraction
        return t

    def t_error(self, t):
        print(("Illegal character '%s'" % t.value[0]))
        t.lexer.skip(1)

    def t_newline(self, t):
        r'\n+'
        t.lexer.lineno += t.value.count("\n")

    ### YACC METHODS ###

    def p_container__LPAREN__DURATION__node_list_closed__RPAREN(self, p):
        r'''container : LPAREN DURATION node_list_closed RPAREN
        '''
        from abjad.tools import rhythmtreetools
        p[0] = rhythmtreetools.RhythmTreeContainer(
            children=p[3],
            preprolated_duration=abs(p[2]),
            )

    def p_error(self, p):
        if p:
            print(("Syntax error at '%s'" % p.value))
        else:
            print("Syntax error at EOF")

    def p_leaf__INTEGER(self, p):
        r'''leaf : DURATION
        '''
        from abjad.tools import rhythmtreetools
        p[0] = rhythmtreetools.RhythmTreeLeaf(
            preprolated_duration=abs(p[1]),
            is_pitched=0 < p[1],
            )

    def p_node__container(self, p):
        r'''node : container
        '''
        p[0] = p[1]

    def p_node__leaf(self, p):
        r'''node : leaf
        '''
        p[0] = p[1]

    def p_node_list__node_list__node_list_item(self, p):
        r'''node_list : node_list node_list_item
        '''
        p[0] = p[1] + [p[2]]

    def p_node_list__node_list_item(self, p):
        r'''node_list : node_list_item
        '''
        p[0] = [p[1]]

    def p_node_list_closed__LPAREN__node_list__RPAREN(self, p):
        r'''node_list_closed : LPAREN node_list RPAREN
        '''
        p[0] = p[2]

    def p_node_list_item__node(self, p):
        r'''node_list_item : node
        '''
        p[0] = p[1]

    def p_toplevel__EMPTY(self, p):
        r'''toplevel :
        '''
        p[0] = []

    def p_toplevel__toplevel__node(self, p):
        r'''toplevel : toplevel node
        '''
        p[0] = p[1] + [p[2]]



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Staff_engraver_consists_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    staff.consists_commands.append('Horizontal_bracket_engraver')
    staff.consists_commands.append('Instrument_name_engraver')

    r'''
    \new Staff \with {
        \consists Horizontal_bracket_engraver
        \consists Instrument_name_engraver
    } {
        c'8
        d'8
        e'8
        f'8
    }
    '''

    assert inspect_(staff).is_well_formed()
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff \with {
            \consists Horizontal_bracket_engraver
            \consists Instrument_name_engraver
        } {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )
